# Implementation notes

## Extraction plugin

The general goal of the extraction plugin is to:

- find marked strings/elements for extraction
- validate the strings/elements
- sanitize elements with unsafe attributes

This is achieved by processing the source through Babel, identifying AST nodes
corresponding to `i18n()` or `<I18N>...</I18N>` and extracting their contents.

### Extracting i18n()

For `i18n()` the process is currently simple: assert that there is exactly one
argument to the function and that it is a `StringLiteral`. All other cases fail.
The single argument will be extracted verbatim.

### Extracting &lt;I18N&gt;

JSX extraction is much more difficult, and the *raison d'être* of jsxlate.

#### Validation

The first step of extraction is to validate the source string. The following
constraints are checked for each message:

- No nesting of `<I18N>` messages is allowed
- Any element with unsafe attributes must have an `i18n-id`
- Multiple `<ReactComponent>`s of the same type must have (distinct) `i18n-id`s
- Only `Identifier` and simple `MemberExpression` nodes are allowed inside
  `JSXExpressionContainer`s

These checks are enforced via a `babel-traverse` path which maintains a context
in order to track all of the names/ids of React Components encountered.

#### Sanitization

By default, the following attributes are whitelisted (tag: [attributes]):

```javascript
whitelistedAttributes: {
    a:   ['href'],
    img: ['alt'],
    '*': ['title', 'placeholder', 'alt', 'summary'],
    'Pluralize': ['on'],
    'Match': ['when'],
},
```

Any attributes for `*` will be merged with attributes for a specific tag or
component. It [should be possible](#extract-todo) to specify these via the
`.babelrc` [plugin options](https://babeljs.io/docs/plugins/#plugin-options),
but this is not yet implemented.

Any attribute not present in the whitelist will be removed.

### Message Output

Finally, `babel-generator` is used on each message AST node after validation
and sanitization. Comments are stripped during this phase. Messages are
collected per source file in `extractFromSource()` and merged across files
in `extractFromPaths()`.
