// DOM Utility Functions
// Get around browser specific oddities.

// Get pixel offset of a node from top of the document.
function offsetTop(node) {
    if (!(node instanceof window.Node)) return;
    var viewPos = (
        // IE (body.scrollTop is always 0, so check documentElement)
        (document.documentElement && document.documentElement.scrollTop) ||
        // Webkit (documentElement.scrollTop is always 0, so use body)
        document.body.scrollTop
    );
    return node.getBoundingClientRect().top + viewPos;
}

// Scroll body to reveal a node in the viewport.
function scrollToReveal(node, offset = 0) {
    if (!node) return;
    window.scrollTo(0, offsetTop(node) + offset);
}

// Viewport scroll distance from top of page
function scrollOffsetTop() {
    var supportPageOffset = window.pageXOffset !== undefined;
    var isCSS1Compat = ((document.compatMode || "") === "CSS1Compat");

    return supportPageOffset ? window.pageYOffset : isCSS1Compat ? document.documentElement.scrollTop : document.body.scrollTop;
}

module.exports = {
    offsetTop,
    scrollToReveal,
    scrollOffsetTop
};
