var _ = require('underscore');
var Immutable = require('immutable');


class FormController {
    initForm(rootComponent) {
        this.state = Immutable.Map();
        this.root = rootComponent;
        this.registerDefaults();
    }

    registerDefaults() {
        ;
    }

    resetForm() {
        this.resetting = true;
        this.initForm(this.root);
        this.resetting = false;
    }

    forceUpdateRoot() {
        if (!this.resetting && this.root) {
            // this.root.forceUpdate() PSYCH.

            // XXX:eo HACK! this should do the above, but that somehow
            // invokes the React browser updates instead of the server
            // side update mechanism. So for now we set a throwaway state key
            // to have the same effect.
            this.root.setState({__thisIsReallyAForceUpdate: true});
        }
    }

    _set(path, val) {
        this.state = this.state.setIn(path, val);
        this.forceUpdateRoot();
    }

    setField(fieldOrPath, val) {
        if (typeof fieldOrPath === 'string') {
            fieldOrPath = [fieldOrPath];
        }
        this._set(fieldOrPath, val);
    }

    setMany(pairs) {
        this.state = this.state.mergeDeep(pairs);
        this.forceUpdateRoot();
    }

    getField(fieldOrPath) {
        if (typeof fieldOrPath === 'string') {
            fieldOrPath = [fieldOrPath];
        }
        return this.state.getIn(fieldOrPath);
    }

    changeHandlerFor(...path) {
        return (evtOrValue) => {
            this.setField(path, evtOrValue.target ? evtOrValue.target.value : evtOrValue);
        };
    }

    getCurrentPane() {
        return this.getField('_currentPane');
    }

    getState() {
        return this.state;
    }

    getStateAsPayload() {
        var type = this.getField('type');
        if (!type) {
            throw new Error('JSON-API spec requires a type.');
        }
        return {
            data: {
                type: type,
                attributes: this.state.toJS()
            }
        };
    }
}


export default FormController;
