var Immutable = require('immutable');
var React = require('react');
var isList = Immutable.List.isList;
var isMap = Immutable.Map.isMap;

var {Schema} = require('conformist');

var IDOPropTypes = {
    immutable: createOptionalValidator(function(props, propName, componentName) {
        var thing = props[propName];
        if (!(isList(thing) || isMap(thing))) {
            return getError('An Immutable type', props, propName, componentName, false);
        }
    }),

    react: {
        children: React.PropTypes.oneOfType([
            React.PropTypes.arrayOf(React.PropTypes.node),
            React.PropTypes.node
        ])
    },

    conformist: createOptionalValidator(function(props, propName, componentName) {
        var thing = props[propName];
        if (!(thing instanceof Schema.Type)) {
            return getError('A Conformist schema', props, propName, componentName, false);
        }
    })
    // TODO: All of the conformist subtypes:
    // conformist.map
    // conformist.list
    // conformist.shape (map with props)
    // conformist.listOf (array with item check)
};

IDOPropTypes.immutable.map = createOptionalValidator(
    function(props, propName, componentName) {
        if (!isMap(props[propName])) {
            return getError('An Immutable Map', props, propName, componentName, false);
        }
    }
);

IDOPropTypes.immutable.list = createOptionalValidator(
    function(props, propName, componentName) {
        if (!isList(props[propName])) {
            return getError('An Immutable List', props, propName, componentName, false);
        }
    }
);


IDOPropTypes.immutable.isRequired = createRequiredType('An immutable', IDOPropTypes.immutable);
IDOPropTypes.immutable.map.isRequired = createRequiredType('A map', IDOPropTypes.immutable.map);
IDOPropTypes.immutable.list.isRequired = createRequiredType('A list', IDOPropTypes.immutable.list);

// An Immutable list is required for OrgIntakeForm.props.value, not type object
// An Immutable list (optional) is expected for OrgIntakeForm.props.value, not type object
function getError(expected, props, propName, componentName, isRequired) {
    var msg;
    if (isRequired) {
        msg = expected + ' is required for ' + componentName +
            '.props.' + propName + ', not type ' + (typeof props[propName]);
    } else {
        msg = expected + ' (optional) is expected for ' + componentName +
            '.props.' + propName + ', not type ' + (typeof props[propName]);
    }
    return new Error(msg + '.');
}

function createRequiredType(expected, validator) {
    return function(props, propName, componentName) {
        if (props[propName] === undefined) {
            return getError(expected, props, propName, componentName, true);
        }
        return validator(props, propName, componentName);
    };
    return validator;
}

function createOptionalValidator(validator) {
    return function(props, propName, componentName) {
        if (props[propName] === undefined) return;
        return validator(props, propName, componentName);
    };
}

module.exports = IDOPropTypes;
