var App = require('dealers/App');
var Router = require('Router');


// Helper for Client-side Navigation

// This mixin expects the component to provide a function from
// clickEvent -> Route and will use client-side navigation
// when appropriate.
module.exports = {
    navigateWithClick: function (e) {
        // cancel clicks on href="#"
        // NB: e.target.href is the full absolute URL that the
        // link refers to.
        var href = e.target.attributes.getNamedItem('href');
        if (href && href.value === '#') {
            e.preventDefault();
        }

        var route = this.route(e);
        if (route
            && this.routeIsDifferentPage(route)
            && Router.routeIsSuitableTargetForClientNavigation(route)
            && this.eventIsPlainOldClick(e)
        )
            {
                e.preventDefault();
                this.world().performAction(App.Navigate(route));
            }
    },

    eventIsPlainOldClick: function (e) {
        return e.button === 0 &&
        ! e.metaKey &&
        ! e.ctrlKey &&
        ! e.shiftKey &&
        ! e.altKey;
    },

    // We ignore, for now, links from non-reacty sources like the
    // content of grad fair pages. (also their nav)
    eventSourceIsReact: function(e) {
        return e.target.getAttribute('data-reactid') !== null;
    },

    // We ignore links to the currently visible page
    // in order to allow the browser to handle fragments.
    routeIsDifferentPage: function (route) {
        var current = Router.routeForWorld(this.world());
        return ! route.equals(current);
    }
};
