"use strict";

var redirect = require('redirect');
var Wish = require('Wish');

/*
    This mixin causes a page component to issue a redirect to an error page
    if any of its initialWishes yield an error.
    The component must also mix in Worldly.
    If it's February 2015 or later and we haven't yet talked about removing
    this, we need to talk about removing this. I think it's just a matter of
    implementing error pages in i5. We're possibly breaking the web by issuing
    302 codes instead of the appropriate error codes. Hard to say.
*/
module.exports = {
    componentWillMount: function () {
        this.constructor.initialWishes(this.world()).forEach(wishFactory => {
            Wish.look(wishFactory(this.world()), {
                Pending: () => null,
                Available: () => null,
                Error: (error) => {
                    var url = {
                        401: '/unauthorized',
                        403: '/forbidden',
                        404: '/not-found'
                    }[error.get('status')] || '/hmm';
                    redirect(this.world(), url);
                }
            });
        });
    }        
};
