/*
    A list of choices from which the user can choose a subset.
    The onChange handler is passed the new subset of choices.
*/
var React = require('react');
var _ = require('underscore');


var Multiselect = React.createClass({
    propTypes: {
        choices: React.PropTypes.arrayOf(React.PropTypes.string).isRequired,
        selectedChoices: React.PropTypes.arrayOf(React.PropTypes.string).isRequired,
        onChange: React.PropTypes.func.isRequired,
        name: React.PropTypes.string.isRequired,
        disabled: React.PropTypes.bool
    },

    render: function () {
        return <ul className="abc-multiselect">
            {this.props.choices.map(this.renderChoice)}
        </ul>;
    },

    renderChoice: function (choice) {
        var isSelected = this.props.selectedChoices.indexOf(choice) != -1;
        return <li key={choice}>
            <label className="checkbox">
                <input type="checkbox"
                       name={this.props.name}
                       checked={isSelected}
                       disabled={this.props.disabled}
                       onKeyDown={this.toggleCheckbox}
                       value={choice}
                       onChange={this.checkboxWasChanged}>
                </input>
                {choice}
            </label>
        </li>;
    },

    toggleCheckbox: function (event) {
        if (event.key == "Enter") {
            event.preventDefault();
            event.target.checked = !event.target.checked;
        }
    },

    checkboxWasChanged: function (event) {
        var newChoices = [];
        var choice = event.target.value;
        if (event.target.checked) {
            newChoices = _.union(this.props.selectedChoices, [choice]);
        } else {
            newChoices = _.without(this.props.selectedChoices, choice);
        }
        this.props.onChange(newChoices);
    }
});

module.exports = Multiselect;
