var I18N = require('./mixins/i18n.js');
var Undo = require('./mixins/undo.js');

/*
    A list of components you can add to or remove from on the fly. Even with
    undo! (Unless you specify otherwise, although it's not 100% clear yet
    how that is done. FIXME:daaaaaave)

    The childComponentFactory function should take two arguments:
    - an item (can be text or whatever else you want), and
    - a function, which this component will provide, that takes one argument -
      the new value of a child component when it has changed - and updates
      this component's state with it. (Presumably you will pass this function
      to the child component you return, as an onChange handler or somesuch.)
    Your factory should return a ReactComponent.

    If that sounds too weird, you probably want ABC.FormFieldSequence anyway.
    Failing that, FormFieldSequence is a good implementation example to start
    from.
*/
var MutableSequence = React.createClass({
    propTypes: {
        childComponentFactory: React.PropTypes.func.isRequired,
        onChange: React.PropTypes.func.isRequired,
        value: React.PropTypes.array.isRequired
    },

    mixins: [ I18N, Undo ],

    errorForItem: function (sequenceIndex) {
        var fieldErrorKey, errorVal;
        if (!_.isUndefined(this.props.model.mutableSequenceItemKey)) {
            fieldErrorKey = this.props.model.mutableSequenceItemKey(this.props.name, sequenceIndex);
            errorMsg = this.props.model.errorMessageForFieldNamed(fieldErrorKey);
            if (!_.isUndefined(errorMsg)) {
                return (
                    <div className="error">{errorMsg}</div>
                );
            }
        }
    },

    shouldShowRemove: function (items) {
        return !(items.length === 1 && items[0].length === 0);
    },

    render: function () {
        var items = this.items();
        if (items.length === 0) {
            items = [''];
        }
        return <fieldset>
            <dl className="abc-mutable-sequence">
                <dt>{this.props.label}</dt>
                {items.map(function(item, index) {
                    return <dd className="sequence-item">
                        {this.props.childComponentFactory(item, _.partial(this.replace, index))}
                        <ABC.If test={this.shouldShowRemove(items)}>
                            <button className="delete remove-sequence-item"
                                    onClick={_.partial(this.remove, index)}>
                                &times;
                            </button>
                        </ABC.If>
                        {this.errorForItem(index)}
                    </dd>;
                }, this)}
            </dl>
            <span className="sequence-actions">
                <a className="add-one"
                    href="#"
                    onClick={this.add}>
                    &#43; {this.gettext('Add another')}
                </a>
            </span>
        </fieldset>;
    },

    items: function () {
        return this.currentValue();
    },

    add: function (evt) {
        evt.preventDefault();
        this.mutate(false, function(value) {
            value.push('');
        });
    },

    remove: function (index, evt) {
        evt.preventDefault();
        this.mutate(true, function(value) {
            value.splice(index, 1);
        });
    },

    replace: function (index, newItem) {
        this.mutate(false, function(value) {
            value[index] = newItem;
        });
    }
});

module.exports = MutableSequence;
