"use strict";

var React = require('react/addons');
var classNames = require('classnames');

var If = require('abc/if');

var PopupMenu = React.createClass({
    getInitialState: function () {
        return {visible: false};
    },

    propTypes: {
        /* A React component that will sit above the menu and remain visible. */
        trigger: React.PropTypes.element.isRequired,

        /* Number of milliseconds to delay the disappearance of the menu when
        the mouse leaves - a grace period during which reentry will cancel the
        hide. Defaults to 600. */
        outDelay: React.PropTypes.number
    },

    getDefaultProps: function() {
        return {
            outDelay: 600,
            className: '',
            dropTab: false
        };
    },

    render: function () {
        var triggerClasses = classNames({
            'drop-tab': this.props.dropTab,
            'delayed-hover-trigger': true
        });
        return <div className="delayed-hover-wrap">
            <div className={triggerClasses}
                  onMouseEnter={this.onMouseEnter}
                  onMouseLeave={this.onMouseLeave}
                  onTouchEnd={this.onTriggerTouch}>
                {this.props.trigger}
            </div>
            <If test={this.state.visible} transition="delayed-hover">
                <div className={this.props.className + ' delayed-hover-body'}
                     onMouseEnter={this.onMouseEnter}
                     onMouseLeave={this.onMouseLeave}>
                    {this.props.children}
                </div>
            </If>
        </div>;
    },

    onMouseEnter: function () {
        this.setState({ visible: true });
        if (this.delayedHoverTimeout) {
            window.clearTimeout(this.delayedHoverTimeout);
            delete this.delayedHoverTimeout;
        }
    },

    onMouseLeave: function () {
        this.delayedHoverTimeout = window.setTimeout(this._close, this.props.outDelay);
    },

    _close: function() {
        this.setState({ visible: false });
    },

    onTriggerTouch: function(event) {
        event.preventDefault();
        if (this.state.visible) {
            this._close();
            document.getElementById('root').removeEventListener('touchend', this._close);
        } else {
            this.onMouseEnter();
            document.getElementById('root').addEventListener('touchend', this._close);
        }
    }
});

module.exports = PopupMenu;
