var _ = require('underscore');
var React = require('react');
var WysiHTMLEditor = require('elements/WysiHtmlEditor');

var {i18n, I18N} = require('I18N');

/*
    A WYSIHTML5 rich text editor.
*/
var RichTextEditor = React.createClass({
    propTypes: {
        // HTML text.
        value: React.PropTypes.string.isRequired,

        // Called when the editor loses focus and frequnetly during text entry
        // (currently whenever a word is entered). Passed the edited HTML text.
        onChange: React.PropTypes.func.isRequired,

        // A WYSIHTML5 configuration object. These can be generated from
        // HTMLFlavor objects on the server.
        wysihtml5Configuration: React.PropTypes.object.isRequired,

        // The URL of a stylesheet to be used from within the iframe itself.
        editorStylesheetUrl: React.PropTypes.string.isRequired
    },

    componentDidMount: function () {
        this.wysihtml5 = this.refs.editor._rte;
        this.wysihtml5.on('change', this.textDidChange);
        this.wysihtml5.on('blur', this.textDidChange);
        this.wysihtml5.on('newword:composer', this.textDidChange);
    },

    textDidChange: function () {
       this.wysihtml5.composer.selection.editor.synchronizer.fromComposerToTextarea(true);
       this.props.onChange(this.refs.editor.getDOMNode().value);
    },

    getValue: function() {
        return this.refs.editor.getDOMNode().value;
    },

    getInitialState: function () {
        return {toolbarId: _.uniqueId("abc-rte-toolbar-")};
    },

    wysihtml5Configuration: function () {
        return this.props.wysihtml5Configuration;
    },

    stylesheetUrl: function () {
        return this.props.editorStylesheetUrl;
    },

    renderToolbar: function () {
        var self = this;
        return this.wysihtml5Configuration().toolbar.map(function (item) {
            return <a data-wysihtml5-command={item.command}
                      key={item.command}
                      title={item.title}
                      href="#"
                      tabIndex="-1">
                <i className={item.icon_class}></i>
            </a>;
        });
    },

    render: function () {
        return <div>
            <div className="rte-container">
                <div className="toolbar" id={this.state.toolbarId} style={{display:'none'}}>
                    {this.renderToolbar()}

                    {/* Link creation/edit dialog */}
                    <div data-wysihtml5-dialog="createLink" className="create-link-dialog" style={{display: 'none'}}>
                        <label><I18N>Link:</I18N>
                            <input data-wysihtml5-dialog-field="href" defaultValue="http://"/>
                        </label>
                        <label><I18N>Text:</I18N>
                            <input data-wysihtml5-dialog-field="text"/>
                        </label>
                        <input type="hidden" data-wysihtml5-dialog-field="rel" defaultValue="nofollow"/>
                        <input type="hidden" data-wysihtml5-dialog-field="target" value="_blank"/>
                        <div className="dialog-buttons">
                            <button data-wysihtml5-dialog-action="save"><I18N>OK</I18N></button>
                            <button data-wysihtml5-dialog-action="cancel" className="cancel"><I18N>Cancel</I18N></button>
                        </div>
                    </div>

                    {/* Edit image dialog */}
                    <div data-wysihtml5-dialog="editImage" className="edit-image-dialog" style={{display: 'none'}}>
                        <label><I18N>Width:</I18N>
                            <input data-wysihtml5-dialog-field="width" className="short-input"/>
                        </label>
                        <label><I18N>Height:</I18N>
                            <input data-wysihtml5-dialog-field="height" className="short-input"/>
                        </label>
                        <label><I18N>Caption:</I18N>
                            <input data-wysihtml5-dialog-field="caption"/>
                        </label>
                        <label><I18N>Link:</I18N>
                            <input data-wysihtml5-dialog-field="href"/>
                        </label>
                        <input type="hidden" data-wysihtml5-dialog-field="rel" defaultValue="nofollow"/>
                        <input type="hidden" data-wysihtml5-dialog-field="target" value="_blank"/>
                        <div className="dialog-buttons">
                            <button data-wysihtml5-dialog-action="save"><I18N>OK</I18N></button>
                            <button data-wysihtml5-dialog-action="cancel" className="cancel"><I18N>Cancel</I18N></button>
                        </div>
                    </div>

                    {/* UploadCare widget */}
                    <div className="uploadcare-button-container">
                        <button className="custom-uploadcare-widget-button" data-resize-image="undo"><I18N>Undo</I18N></button>
                        <button className="custom-uploadcare-widget-button" data-resize-image="complete"><I18N>Complete</I18N></button>
                        <div className="uploadcare-dialog-title"><I18N>Resize Image</I18N></div>
                    </div>

                    {/* YouTube embed dialog */}
                    <div data-wysihtml5-dialog="embedYoutube" className="add-video-dialog" style={{display: 'none'}}>
                        <label><I18N>URL:</I18N>
                            <input data-wysihtml5-dialog-field="src"/>
                        </label>
                        <label><I18N>Width:</I18N>
                            <input data-wysihtml5-dialog-field="width"/>
                        </label>
                        <div className="dialog-buttons">
                            <button data-wysihtml5-dialog-action="save"><I18N>OK</I18N></button>
                            <button data-wysihtml5-dialog-action="cancel" className="cancel">{i18n('Cancel')}</button>
                        </div>
                    </div>
                </div>

                <WysiHTMLEditor
                    ref="editor"
                    value={this.props.value}
                    onChange={function() {}} // Use a no-op onChange to suppress React's warning, because React
                                                   // doesn't see that we are listening for changes using WYSIHTML5's events.
                    toolbarId={this.state.toolbarId}
                    parserRules={JSON.stringify(this.wysihtml5Configuration())}
                    stylesheetUrl={this.stylesheetUrl()} />
            </div>
        </div>;
    },
});

module.exports = RichTextEditor;
