/*
    A list of tokens which the user can add to and remove from.
*/
var TokenEditor = React.createClass({
    propTypes: {
        name: React.PropTypes.string.isRequired,
        disabled: React.PropTypes.bool,
        // Optional text at top
        prompt: React.PropTypes.string,
        // Default text for the TokenField to hint the user as to what they
        // should type.
        placeholder: React.PropTypes.string,
        // An array of user-defined tokens.
        tokens: React.PropTypes.arrayOf(React.PropTypes.string).isRequired,
        // Change handler which takes the list of user defined tokens
        onChange: React.PropTypes.func.isRequired
    },

    getDefaultProps: function () {
        return {prompt: ''};
    },

    render: function() {
        return <div className={this.props.className}>
                <ABC.If test={this.props.prompt.length > 0}>
                    <label>{this.props.prompt}</label>
                </ABC.If>
                <TokenField
                    name={this.props.name}
                    tokens={this.props.tokens}
                    onChange={this.props.onChange}
                    disabled={this.props.disabled}
                    placeholder={this.props.placeholder} />
                <TokenList
                    tokens={this.props.tokens}
                    name={this.props.name}
                    onChange={this.props.onChange}
                    disabled={this.props.disabled} />
            </div>;
    }
});

// Do not export this in Browserify, only internally used
var TokenField = React.createClass({

    render: function() {
        return <input
                ref="input"
                name={this.props.name}
                placeholder={this.props.placeholder}
                onKeyDown={this.addToken}
                disabled={this.props.disabled} />;
    },

    addToken: function(event) {
        if (event.key == 'Enter') {
            event.preventDefault();
            // regex trims beginning/end white space and brings internal whitespaces to 1
            var val = event.target.value.replace(/^\s+|\s+$/g, '').replace(/\s+/g, ' '),
                tokens = this.props.tokens;
            if (val && tokens.indexOf(val) == -1) {
                tokens = tokens.concat(val);
            }
            event.target.value = '';
            this.props.onChange(tokens);
        }
    }
});

// Do not export this in Browserify, only internally used
var TokenList = React.createClass({
    render: function() {
        return <ul>
            {this.renderItems(this.props.tokens)}
        </ul>;
    },

    renderItems: function(tokens) {
        return tokens.map(function(token) {
            return <li key={token}
                       data-token-item={token}
                       onClick={this.removeToken}>{token}</li>;
        }.bind(this));
    },

    removeToken: function(evt) {
        if (!this.props.disabled) {
            this.props.onChange(_.without(this.props.tokens, evt.target.getAttribute('data-token-item')));
        }
    }
});


module.exports = TokenEditor;
