/*
  A thin wrapper around jQuery UI autocomplete.
*/
var Typeahead = React.createClass({
    propTypes: {
        defaultValue: React.PropTypes.string,

        // The number of characters typed before autocompletion starts.
        minimumLength: React.PropTypes.number,

        // A function that is passed an item (as returned by dataSource,
        // below) and returns a jQuery node. The node must be an <li>
        // and must contain an <a>.
        jqueryNodeForItem: React.PropTypes.func.isRequired,

        // A function that is passed (term, callback) where term is the
        // string the user has typed so far, and callback should be passed
        // an array of objects each containing a 'value' property and any
        // other information needed in jqueryNodeForItem, above.
        dataSource: React.PropTypes.func.isRequired,

        // A function that is called when the user selects an item from
        // the menu or when focus leaves the control. It is passed
        // (newValue, wasSelectedFromMenu), where wasSelectedFromMenu
        // is a boolean indicating whether a menu choice was selected or
        // whether the user simply typed something and then blurred
        // the control. If the user selected something from the menu,
        // then newValue is the 'value' property of the corresponding item
        // as yielded by dataSource, above. Otherwise, 'newValue' is
        // whatever the user typed before leaving the control.
        onChange: React.PropTypes.func.isRequired,

        // While fetching, a spinner may be displayed in the list of results.
        // NOTE: The spinner is added to the typeahead as the last item in
        // the results list and hidden/shown based on pending requests. Any
        // calculations on number of results when 'spins' is true should take
        // this into account.
        spins: React.PropTypes.bool
    },

    getDefaultProps: function() {
        return {
            defaultValue: '',
            minimumLength: 2,
            spins: false
        };
    },

    statics: {
        /*
          Given a search result item (string) and search query (string),
          return a string for the search result with <strong> tags added
          that highlight the parts of the item that match the search query.
          Matches by lower case.

          XXX:max This was built for use in the org intake deduper, but it's
          not the right solution for that, since the highlighting behavior
          doesn't at all correspond with Xapian's search. I suggest removal.
        */
        stringByBoldingMatchingPart: function(item, query) {
            item = item.split("");
            query = query.split("");

            // First assign to each character bold or not bold:
            result = [];
            nextQueryChar = query.shift();
            while(!_.isEmpty(item)) {
                nextItemChar = item.shift();
                if(nextItemChar.toLocaleLowerCase() == nextQueryChar.toLocaleLowerCase()) {
                    nextQueryChar = query.shift() || '';
                    result.push({str: nextItemChar, bold: true});
                } else {
                    result.push({str: nextItemChar, bold: false});
                }
            }

            // Now insert tags at the boundaries:
            var secondResult = [];
            var one, two = {str: '', bold: false};
            while(!_.isEmpty(result)) {
                one = two;
                two = result.shift();
                if ( ! one.bold && two.bold ) {
                    secondResult.push("<strong>" + two.str);
                } else if( one.bold && ! two.bold ) {
                    secondResult.push("</strong>" + two.str);
                } else {
                    secondResult.push(two.str);
                }
            }
            if (two.bold) {
                secondResult.push("</strong>");
            }
            return secondResult.join("");
        }
    },

    render: function () {
        return <div>
        <input ref='input'
        onFocus={this.showTypeahead}
        disabled={this.props.disabled ? 'disabled' : ''}
        defaultValue={this.props.defaultValue}
        onKeyDown={this.firstResultOnEnter}/>
        </div>;
    },

    componentDidMount: function() {
        var input = $(this.refs.input.getDOMNode());
        input.autocomplete({
            source: this.autocompleteDataSource,
            minLength: this.props.minimumLength,
            appendTo: input.parent()
        })
        // FIXME: can we debounce for just one event loop?
        .on('autocompleteselect autocompletechange', _.debounce(this.handleSelection, 50))
        .data('ui-autocomplete')._renderItem = this.autocompleteRenderItem;
        input.autocomplete('widget').addClass('org-deduper');

        if (this.props.spins && !this.spinner) {
            this.setupSpinner();
        }
    },

    componentWillReceiveProps: function(newProps) {
        if (!this.spinner && newProps.spins) {
            this.setupSpinner();
        }
    },

    componentWillUnmount: function() {
        $(this.refs.input.getDOMNode()).autocomplete("destroy");
        if (this.spinner) {
            this.spinner.stop();
        }
    },

    showTypeahead: function () {
        var input = $(this.refs.input.getDOMNode());
        input.autocomplete('search', input.val());
    },

    setupSpinner: function() {
        var input = $(this.refs.input.getDOMNode());
        this.spinner = new Spinner({
            length: 4,
            radius: 7,
            width: 3,
            color: '#999'
        });
        input.on('autocompletesearch', this.startSpinner)
        .on('autocompleteresponse', this.stopSpinner);
    },

    spinnerOverlay: function() {
        var input = $(this.refs.input.getDOMNode());
        var spinnerLi = input.next().find('.spinner-overlay');
        return spinnerLi[0];
    },

    forceOpenWithSpinner: function() {
        var $input = $(this.refs.input.getDOMNode());

        if ($input.next().is(":visible")) {
            // Bail if we already have results to spin over.
            return;
        }

        // jQuery UI Autocomplete doesn't provide a way to open the
        // menu via the API, so we dig a bit and do it ourselves.
        var widget = $input.data('uiAutocomplete');
        widget._suggest([{
            name: '', value: '', term: '', isExact: null, isSpinner: true}
        ]);
    },

    startSpinner: function(event, ui) {
        this.forceOpenWithSpinner();
        var overlay = this.spinnerOverlay();
        this.spinner.spin(overlay);
        overlay.style.display = 'block';
    },

    stopSpinner: function(event, ui) {
        this.spinner.stop();
        this.spinnerOverlay().style.display = 'none';
    },

    autocompleteDataSource: function (request, response) {
        this.props.dataSource(request.term, function (results) {
            if(_.contains(_.pluck(results, 'value'), undefined)) {
                throw new Error(
                    "ABC.Typeahead's dataSource prop must yield objects with " +
                    "a 'value' property." +
                    ABC.getDeclarationErrorAddendum(this));
            }

            if (this.props.spins) {
                // Style a spinner over the entire list of results and
                // disable interaction while fetching more results.
                results.push({
                    name: '',
                    value: '',
                    term: '',
                    isExact: null,
                    isSpinner: true
                });
            }
            response(results);
        }.bind(this));
    },

    autocompleteRenderItem: function (ul, item) {
        var node = this.props.jqueryNodeForItem(item);
        if (!node.is || !node.is("li")) {
            throw new Error(
                "ABC.Typeahead's jqueryNodeForItem prop must return a jQuery <li> node." +
                ABC.getDeclarationErrorAddendum(this));
        }
        if (!node.has("a").length) {
            throw new Error(
                "The jQuery node returned by ABC.Typeahead's jqueryNodeForItem prop " +
                "must contain an <a> somewhere in it. Because jQuery!" +
                ABC.getDeclarationErrorAddendum(this));
        }

        if (item.isSpinner) {
            node.addClass('spinner-overlay');
        }
        return node.appendTo(ul);
    },

    firstResultOnEnter: function(event) {
        if (event.key == 'Enter' || event.key == 'Tab') {
            event.preventDefault();
            var results = $(this.refs.input.getDOMNode()).next().children();
            if (results.length) {
                var firstResult = results.not('.spinner-overlay').first();
                this.handleSelection(null, {
                    item: {
                        // fool handleSelection with a faux ui.item.value
                        value: firstResult.data('uiAutocompleteItem').value
                    }
                });
            }
        }
    },

    handleSelection: function(event, ui) {
        // ui.item exists if the user selected a choice from the menu.
        // If not, the user simply blurred the input without choosing
        // something. In this case, we use whatever value the user
        // typed into the input field before blurring it.
        var wasSelectedFromMenu = !!ui.item;
        var newValue = wasSelectedFromMenu ?
        ui.item.value :
        event.target.value;
        $(this.refs.input.getDOMNode()).val(newValue);
        this.props.onChange(newValue, wasSelectedFromMenu);
    }
});

module.exports = Typeahead;
