"use strict";

var Immutable = require('immutable');

var API = require('API');
var Router = require('Router');

var AnalyticsDealer = require('dealers/AnalyticsDealer');

var App = {
    currentUser: function(world) {
        return API.get(world, ['App', 'User'],
            Router.route("user"));
    },

    currentAdministeredAssets: function(world) {
        return API.get(world, ['App', 'AdministeredAssets'],
            Router.route("user/assets"));
    },

    Navigate: function(route) {
        return {
            name: "App/Navigate",
            transaction: (ws) => ws.set('Route', route),
            effect: function(world) {
                var url = Router.urlForRoute(route);
                if (window.history.pushState) {
                    window.history.pushState(route.toJS(), "", url);
                    AnalyticsDealer.trackPageview(url);
                } else {
                    window.location = url;
                }
            }
        };
    },

    SetRouteWithoutNavigating: function(route) {
        return {
            name: "App/SetRouteWithoutNavigating",
            transaction: (ws) => ws.set('Route', route)
        };
    },

    initializeBrowserHistory: function(world) {
        // The history entry associated with the initial page load has no
        // state object associated with it. We provide one so that we can
        // use the back button to get to the initially loaded page:
        if (window.history.replaceState) {
            window.history.replaceState(
                Router.routeForWorld(world).toJS(), '',
                document.location);
        }

        window.addEventListener("popstate", (event) => {
            // event.state will be undefined when some browsers fire this event
            // on the initial page load:
            if (event.state) {
                world.performAction(App.SetRouteWithoutNavigating(
                    Immutable.fromJS(event.state)));
            }
        });
    }
};

module.exports = App;
