"use strict";

var queryString = require('query-string');
var Immutable = require('immutable');

var UserAgent = require('dealers/UserAgent');

// TODO: rename, possibly Browser or BrowserInfo or BrowserDimensions?
var Client = {
    setDimensions(world, dims) {
        world.performAction({
            name: 'Client/SetDimensions',
            transaction: (ws) => ws.setIn(['Client', 'Dimensions'], Immutable.Map(dims))
        });
    },

    getDimensions(world) {
        return world.getIn(['Client', 'Dimensions']);
    },

    isMobile(world, threshold) {
        var dimensions = this.getDimensions(world);
        threshold = threshold || 768;
        // assume we are not mobile on the server (for now)
        return dimensions ? dimensions.get('width') < threshold : undefined;
    },

    browserClasses() {
        let {name, platform, versionNumber, version, ...classes} = UserAgent();
        return {
            ['v' + versionNumber]: true,
            ...classes
        };
    },

    queryParams(world) {
        if (global.document) {
            return Immutable.fromJS(queryString.parse(document.location.search));
        } else {
            if (!world) {
                throw new Error("You must supply a world to Client.queryParams on the server")
            }
            return world.getIn(['Route', 'additionalParameters']);
        }
    }
};

module.exports = Client;
