"use strict";

var _ = require('underscore');
var Immutable = require('immutable');
var superagent = require('superagent');

var API = require('API');
var Router = require('Router');


// TODO: config'urize
//var urlPrefix = 'http://en.geo.idealist.org/';
var urlPrefix = '/_data/en.geo/';


var memoizedAutocompResultsByPrefix = {};


function objectFromGeoTuple(tuple) {
    return {code: tuple[0], name: tuple[1], geoid: tuple[2]};
}

function objectsFromGeoTuples(tuples) {
    return Immutable.List(tuples.map(objectFromGeoTuple));
}


var Geo = {
    /*
    A special value that represents the region of a location in a country which
    has no regions. So, if the user chooses United States for the country
    and makes no region choice, the location value will have 'undefined' for its
    region; but, if the user chooses Antarctica for the country, which has no
    regions, then the location value will have NotApplicable for its region.
    This allows us to distinguish these two cases in validation.
    */
    NotApplicable: -1,

    autocompleteResults: _.debounce(function (prefix, options = {}, cb) {
        var timestamp = options.time;
        var memoizationKey = prefix + (options.type || '');

        if (memoizedAutocompResultsByPrefix[memoizationKey]) {
            cb(memoizedAutocompResultsByPrefix[memoizationKey], {time: timestamp});
        } else {
            superagent
                .get(Router.urlForRoute(Router.route('geo/search',
                        {lang: options.lang || 'en', type: options.type, q: prefix})))
                .end((error, response) => {
                    memoizedAutocompResultsByPrefix[memoizationKey] = response.body.result;
                    cb(response.body.result, {time: timestamp});
                });
        }
    }, 200),

    readRegionsForCountry: (world, countryGeoid) => world.getIn(
        ['Geo', 'regionsForCountry', countryGeoid]),

    getCountries: function(locale, cb) {
        superagent
            .get(Router.urlForRoute(Router.route('geo/countries', {lang: locale || 'en'})))
            .end((error, response) =>
                cb(objectsFromGeoTuples(response.body.result)));
    },

    getRegionsForCountry: (world, countryGeoid) => API.get(world,
        ['Geo', 'regionsForCountry', countryGeoid],
        Router.route('geo/regions', {lang: world.get('Locale'), country_geoid: countryGeoid}),
        response => objectsFromGeoTuples(response.get('result').toJSON())
    )
};

module.exports = Geo;
