var _ = require('underscore');

var Actions = require('Actions');
var Client = require('dealers/Client');

var afterClosePath = ['LoginSignup', 'afterClose'];
var signupPath = ['LoginSignup', 'isOpen'];


module.exports = {
    openSignup(world, afterCloseAction) {
        world.performAction({
            name: 'LoginSignup/openSignup',
            transaction: (ws) => ws.setIn(signupPath, true).
                                    setIn(afterClosePath, afterCloseAction),
            effect: (world) => {
                // TODO: find a better home for these, maybe like Client.scrollToFooter()?
                if (Client.isMobile(world)) {
                    _.defer(() => document.querySelector('footer .signup-flow').scrollIntoView());
                }
            }
        });
    },

    closeSignup(world) {
        // At this point, we have crossed an authentication boundary
        // or the user has explicitly exited from the Signup flow.
        var afterCloseAction = this.afterCloseAction(world);
        if (afterCloseAction) {
            world.performAction(afterCloseAction);
        }

        world.performAction({
            name: 'LoginSignup/closeSignup',
            transaction: (ws) => ws.setIn(signupPath, false).
                                    removeIn(afterClosePath),
            effect: (world) => {
                // TODO: find a better home for these, maybe like Client.scrollToTop()?
                if (Client.isMobile(world)) {
                    window.scrollTo(0, 0);
                }
            }
        });
    },

    isSignupOpen(world) {
        return world.getIn(signupPath);
    },

    isModalSignupOpen(world) {
        return !Client.isMobile(world) && this.isSignupOpen(world);
    },

    isFooterSignupOpen(world) {
        return Client.isMobile(world) && this.isSignupOpen(world);
    },

    afterCloseAction(world) {
        var action = world.getIn(afterClosePath);
        return Actions.construct(action);
    }
};
