"use strict";

var Immutable = require('immutable');
var config = require('config');

var Cookie = require('dealers/Cookie');


var SystemMessages = {
    messageListFromCookieString: function(cookie) {
        if (!cookie) {
            return Immutable.List([]);
        } else {
            var decoded = new Buffer(cookie, 'base64').toString();
            return Immutable.fromJS(JSON.parse(decoded).map((c) => ({
                title: c[0],
                text: c[1],
                style: c[2],
                isSticky: c[3],
                cookieMarshalled: true
            })));
        }
    },

    all: function(world) {
        return world.getIn(['SystemMessages', 'messages']);
    },

    RemoveEmailVerificationCookie: function() {
        return {
            name: "SystemMessages/RemoveEmailVerificationCookie",
            effect: () => {
                Cookie.expire('i3_pending_verification');
            }
        };
    },

    Dismiss: function(message) {
        return {
            name: "SystemMessages/Dismiss",
            transaction: (worldState) =>
                worldState.updateIn(['SystemMessages', 'messages'], (messages) =>
                    messages.filter((m) => !Immutable.is(m, message)))
        };
    },

    DismissEmailVerification: function() {
        return {
            name: "SystemMessages/DismissEmailVerification",
            transaction: ws =>
                ws.setIn(['SystemMessages', 'emailDismissed'], true)
        };
    }
};

module.exports = SystemMessages;
