'use strict';

var React = require('react');

var Input;
var TextArea;

var baseFontSize;

// TODO:bd make this a wrapper component, not a func

function getPixelSize(size) {
  if (size.indexOf('rem') !== -1) {
    return Math.floor(parseFloat(size) * baseFontSize);
  } else {
    // TODO, maybe, 'em'
    return parseInt(size);
  }
}

function createPlaceholderComponent(DOMComponent) {
  // return this
  var DOMComponentPlaceholder = React.createClass({
    propTypes: {
      placeholder: React.PropTypes.any,
      onClick: React.PropTypes.func,
      onChange: React.PropTypes.func
    },

    componentDidMount: function() {
      // HACK: This is a way to inherit the child input width using the raw
      // CSS value. This allows the shim to be a drop-in replacement for input.
      // It might need more styles later. ONLY IE SHOULD USE THIS
      let node = this.refs.input.getDOMNode();
      let display = node.style.display;
      node.style.display = 'none';
      let width = window.getComputedStyle(node).getPropertyValue('width');
      node.style.display = display; // return to original display prop
      this.getDOMNode().style.width = width;

      this.updatePlaceholder();
    },

    componentDidUpdate: function() {
      this.updatePlaceholder();
    },

    updatePlaceholder: function() {
      if (!this.props.placeholder) {
        return;
      }
      baseFontSize = document.body.currentStyle
        ? parseFloat(document.body.currentStyle.fontSize)
        : 16;
      var input = React.findDOMNode(this.refs.input);
      var placeholder = React.findDOMNode(this.refs.placeholder);
      if (input.value !== '') {
        placeholder.style.display = 'none';
        return;
      }
      placeholder.style.display = 'block';

      // we'll suppose this is ie, which stores css props in `currentStyle`
      var inputStyle = input.currentStyle;

      if (inputStyle) {
        var topOffset = (getPixelSize(inputStyle.marginTop) || 0) +
          (getPixelSize(inputStyle.paddingTop) || 0) +
          (getPixelSize(inputStyle.borderTopWidth) || 0);

        var leftOffset = (getPixelSize(inputStyle.marginLeft) || 0) +
          (getPixelSize(inputStyle.paddingLeft) || 0) +
          (getPixelSize(inputStyle.borderLeftWidth) || 0);

        var fontFamily = inputStyle.fontFamily;
        var fontWeight = inputStyle.fontWeight;
        var fontVariant = inputStyle.fontVariant;
        var fontSize = inputStyle.fontSize;
        var fontStyle = inputStyle.fontStyle;

        var placeholderStyle = placeholder.style;
        placeholderStyle.top = topOffset + 'px';
        placeholderStyle.left = leftOffset + 'px';
        placeholderStyle.fontFamily = fontFamily;
        placeholderStyle.fontWeight = fontWeight;
        placeholderStyle.fontVariant = fontVariant;
        placeholderStyle.fontSize = fontSize;
        placeholderStyle.fontStyle = fontStyle;
      }
    },

    handlePlaceholderClick: function(a, b, c) {
      React.findDOMNode(this.refs.input).focus();
      this.props.onClick && this.props.onClick(a, b, c);
    },

    handleChange: function(a, b, c) {
      // If `defaultValue` is used (aka, no `value` + change handling) then
      // force update to trigger `updatePlaceholder`.
      this.forceUpdate(function() {
        this.props.onChange && this.props.onChange(a, b, c);
      }.bind(this));
    },

    render: function() {
      var wrapperStyle = {position: 'relative', display: 'inline-block'};
      var placeholderStyle = {
        position: 'absolute',
        color: '#999999'
      };

      if (!this.props.placeholder) {
        return <DOMComponent ref="input" {...this.props} />;
      }
      return (
        <div style={wrapperStyle}>
          <DOMComponent ref="input" onChange={this.handleChange} { ...this.props } />
          <div
            ref="placeholder"
            onClick={this.handlePlaceholderClick}
            style={placeholderStyle}>
              {this.props.placeholder}
          </div>
        </div>
      );
    }
  });

  return DOMComponentPlaceholder;
}

var InputWrapper = React.createClass({
  render() {
    return <input {...this.props}/>;
  }
});

var TextAreaWrapper = React.createClass({
  render() {
    return <textarea {...this.props}/>;
  }
});

// module.exports = {
//   Input: createPlaceholderComponent(React.DOM.input),
//   TextArea: createPlaceholderComponent(React.DOM.textarea)
// }

var Input = InputWrapper;
var TextArea = TextAreaWrapper;

if (global.document) {
  var testNode = document.createElement('input');
  var hasPlaceHolderAttr = 'placeholder' in testNode;

  if (!hasPlaceHolderAttr) {
    Input = createPlaceholderComponent(InputWrapper);
    TextArea = createPlaceholderComponent(TextAreaWrapper);
  }
}

module.exports = {
  Input,
  TextArea
};
