"use strict";

var React = require('react');
var _ = require('underscore');
var moment = require('moment');

var {I18N, i18n} = require('I18N');

var BoxWidget = require('elements/BoxWidget');
var ContentGroup = require('elements/ContentGroup');
var If = require('abc/if');
var LinkedMap = require('elements/LinkedMap');


var GradFairDetails = React.createClass({
    statics: {
        accessibilities: {
            'guide_dogs': <li key="guide_dogs"><I18N>Guide dogs are welcome</I18N></li>,
            'wheelchair_access': <li key="wheelchair_access"><I18N>There is wheelchair access</I18N></li>
        }
    },

    render: function() {
        var doc = this.props.fair;

        var address = doc.addresses[0] || {city: ''};

        return <BoxWidget headline="Details">
            <ContentGroup headline="Date and time">
                <p><I18N>Fair starts:</I18N> {moment(doc.utc_start).format('MMMM Do, YYYY, hh:mm a')}</p>
                <p><I18N>Fair ends:</I18N> {moment(doc.utc_end).format('MMMM Do, YYYY, hh:mm a')}</p>
            </ContentGroup>

            <ContentGroup headline="Location">
                <p>{doc.location}</p>
                    <LinkedMap
                        location={address}
                        width={228}
                        height={228}/>
            </ContentGroup>

            {this.props.children}

            <If test={doc.access_other}>
                <ContentGroup headline="Accessibility">
                    <p><I18N>Statement of inclusion:</I18N> {doc.access_other}</p>
                <If test={doc.access}>
                    <I18N>In this venue:</I18N>
                    <ul>{_.map(doc.access, this.labelForAccessibility)}</ul>
                </If>
                </ContentGroup>
            </If>
            <If test={doc.fair_email}>
                <ContentGroup headline="Contact">
                    <p><I18N>Got questions? Email us at: {doc.fair_email}</I18N></p>
                </ContentGroup>
            </If>
        </BoxWidget>
    },

    labelForAccessibility: function (token) {
        return GradFairDetails.accessibilities[token];
    }

});

module.exports = GradFairDetails;
