"use strict";

var React = require('react');
var _ = require('underscore');

var If = require('abc/if');

var uriTemplate = require('uri-templates');
var staticImageUrlTemplate = uriTemplate('http://maps.google.com/maps/api/staticmap{?center,size,zoom,markers,sensor,maptype}');
var linkUrlTemplate = uriTemplate('http://maps.google.com/maps{?q}');

/**
 * Static image on listing pages, linked to Google Maps page
 */
var LinkedMap = React.createClass({
    propTypes: {
        location: React.PropTypes.shape({
            latitude: React.PropTypes.number,
            longitude: React.PropTypes.number
        }),
        zoom: React.PropTypes.number,
        width: React.PropTypes.number,
        height: React.PropTypes.number
    },

    getDefaultProps: function() {
        return {
            zoom: 14,
            width: 228,
            height: 228
        };
    },

    latLng: function () {
        return this.props.location.latitude + ',' + this.props.location.longitude;
    },

    staticImageUrl: function() {
        return staticImageUrlTemplate.fillFromObject({
            center: this.latLng(),
            size: this.props.width + 'x' + this.props.height,
            zoom: this.props.zoom,
            markers: 'markers=color:blue|label:A|' + this.latLng(),
            sensor: false,
            maptype: 'roadmap'
        });
    },

    linkUrl: function() {
        return linkUrlTemplate.fillFromObject({q: this.latLng()});
    },

    render: function() {
        return (
            <If test={!_.isUndefined(this.props.location.latitude) && !_.isUndefined(this.props.location.longitude)}>
                <a href={this.linkUrl()}
                   rel="external"
                   target="_blank">
                    <img src={this.staticImageUrl()}
                         width="100%"/>
                </a>
            </If>
        );
    }
});

module.exports = LinkedMap;
