"use strict";

var React = require('react');
var Immutable = require('immutable');
var {I18N} = require('I18N');

var Geo = require('dealers/Geo');

var SubField = require('elements/Forms/SubField');
var CountryList = require('./CountryList');
var RegionList = require('./RegionList');


var LocationFallback = React.createClass({
    propTypes: {
        value: React.PropTypes.object.isRequired,
        onReset: React.PropTypes.func.isRequired,

        onChange: React.PropTypes.func.isRequired,
        element: React.PropTypes.object.isRequired,
        showErrors: React.PropTypes.bool
    },

    getDefaultProps() {
        return { showErrors: false };
    },

    render: function() {
        return <div className="fallback-form">
            <fieldset>
                <SubField type={CountryList}
                    label={<I18N>Country:</I18N>}
                    onChange={this.countryChanged}
                    showErrors={this.props.showErrors}
                    ref="country"
                    value={this.props.value.get('countryGeoid')}
                    validation={this.props.validation && this.props.validation.children.country}
                    element={this.props.element && this.props.element.members.countryGeoid} />
                <SubField type={RegionList}
                    label={<I18N>Region:</I18N>}
                    onChange={this.regionChanged}
                    value={this.props.value.get('regionGeoid')}
                    showErrors={this.props.showErrors}
                    ref="region"
                    countryGeoid={this.props.value.get('countryGeoid')}
                    validation={this.props.validation && this.props.validation.children.region}
                    element={this.props.element && this.props.element.members.regionGeoid} />
                <SubField type="text"
                    label={<I18N>City:</I18N>}
                    onChange={this.cityChanged}
                    value={this.props.value.get('city')}
                    showErrors={this.props.showErrors}
                    validation={this.props.validation && this.props.validation.children.city}
                    element={this.props.element && this.props.element.members.city} />
                <a href="#" style={{float: 'right'}} onClick={this.props.onReset}>
                    <I18N>Enter a different location</I18N>
                </a>
            </fieldset>
        </div>
    },

    updateValue: function(update) {
        this.props.onChange(this.props.value.merge(update));
    },

    cityChanged: function(city) {
        this.updateValue({city});
    },

    countryChanged: function(country) {
        this.updateValue({
            countryGeoid: country.geoid,
            regionGeoid: undefined,
            countryCode: country.code,
            regionCode: undefined,
            countryName: country.name,
            regionName: undefined
        });
    },

    regionChanged: function(region) {
        if (Immutable.is(region, Geo.NotApplicable)) {
            this.updateValue({
                regionGeoid: Geo.NotApplicable,
                regionCode: '',
                regionName: ''
            });
        } else {
            this.updateValue({
                regionGeoid: region.geoid,
                regionCode: region.code,
                regionName: region.name
            });
        }
    }
});

module.exports = LocationFallback;
