"use strict";

var _ = require('underscore');
var React = require('react');
var Immutable = require('immutable');

var Geo = require('dealers/Geo');

var World = require('World');
var Wish = require('Wish');

var GeoLocationList = require('./GeoLocationList');


var RegionList = React.createClass({
    mixins: [World.Child],

    propTypes: {
        // 'value' may be a geoid OR Geo.NotApplicable OR undefined:
        value: React.PropTypes.number,
        onChange: React.PropTypes.func.isRequired,
        countryGeoid: React.PropTypes.number
    },

    handleOptions: function (list) {
        // Our value becomes NotApplicable when we become the region list for a
        // country with no regions.
        if (this.props.value !== Geo.NotApplicable && list.size === 0) {
            _.defer(() => this.props.onChange(Geo.NotApplicable));
        }
        return list;
    },

    render: function() {
        var options = this.props.countryGeoid === undefined ? Immutable.List() :
            Wish.look(Geo.getRegionsForCountry(this.world(), this.props.countryGeoid), {
                Pending: () => Immutable.List(),
                Error: () => Immutable.List(),
                Available: this.handleOptions
            });
        var value = Immutable.is(this.props.value, Geo.NotApplicable) ? undefined : this.props.value; // [1]
        return <GeoLocationList value={value} options={options} onChange={this.onChange} />

        // [1] There's no user-interface difference between undefined and NotApplicable, so
        // we let GeoLocationList think that a NotApplicable value is actually undefined.
    },

    onChange: function(geoid) {
        var regionsWish = Geo.readRegionsForCountry(this.world(), this.props.countryGeoid);
        if (regionsWish.get('state') !== 'Available') {
            return;
        }
        var selected = regionsWish.get('value').filter(val => val.geoid === geoid);
        this.props.onChange(selected.first());
    }
});

module.exports = RegionList;
