"use strict";

var cuid = require('cuid');
var Immutable = require('immutable');
var React = require('react');

var Geo = require('dealers/Geo');
var World = require('World');
var {I18N, i18n} = require('I18N');

var LocationFallback = require('./LocationFallback');
var LocationTypeahead = require('./LocationTypeahead');
var FallbackPrompt = require('./FallbackPrompt');


var LocationField = React.createClass({
    propTypes: {
        value: React.PropTypes.object.isRequired,
        onChange: React.PropTypes.func.isRequired,
        onSelect: React.PropTypes.func,
        onKeyUp: React.PropTypes.func,
        onFocus: React.PropTypes.func,
        onBlur: React.PropTypes.func,
        validation: React.PropTypes.object,
        element: React.PropTypes.object,
        className: React.PropTypes.string,
        allowFallback: React.PropTypes.bool,
        tabIndex: React.PropTypes.string,
        maxMenuHeight: React.PropTypes.number
    },

    getDefaultProps: function() {
        return {
            allowFallback: true,
            className: '',
            autocompleteEndpointOptions: {}
        };
    },

    mixins: [World.Child],

    statics: {
        blankLocation: function(overrides) {
            return Immutable.Map({
                fallback: false,
                countryGeoid: undefined,
                regionGeoid: undefined,
                countryCode: undefined,
                regionCode: undefined,
                countryName: undefined,
                regionName: undefined,
                city: undefined,
                location: undefined,
                geoid: undefined
            }).merge(overrides);
        }
    },

    getInitialState: function(){
        return {
            currentSearchPrefix: '' // this is only for switching to fallback
        };
    },

    yesFallback: function(event) {
        event.preventDefault();
        var bestGuessCity = this.state.currentSearchPrefix.split(',')[0];
        var location = LocationField.blankLocation(
            {fallback: true, city: bestGuessCity});
        this.props.onSelect && this.props.onSelect(location);
        this.props.onChange && this.props.onChange(location);
    },

    noFallback: function(event) {
        event.preventDefault();
        this.setSearchInput('');
        let blankVal = LocationField.blankLocation();
        this.props.onSelect && this.props.onSelect(blankVal);
        this.props.onChange && this.props.onChange(blankVal);
        // If we're calling this from fallbackWasReset, then refs.typeahead
        // doesn't exist, so we need to wait until it does.
        setTimeout(() => this.refs.typeahead.refs.source.focusInput(), 1);
    },

    fallbackWasReset: function(event){
        this.noFallback(event);
    },

    fetchLocations: function(prefix, options, cb) {
        var locale = this.world().get('Locale');
        if (prefix === '') {
            cb({});
        }
        else {
            Geo.autocompleteResults(prefix,
                Object.assign({lang: locale}, options),
                cb);
        }
    },

    setLocationIsUnrecognized: function(isUnrecognized) {
        this.world().performAction({
            name: 'setLocationIsUnrecognized',
            transaction: (worldState) =>
                worldState.setIn(
                    ['Geo', 'locationIsUnrecognized', this._guid], isUnrecognized)
        });
    },

    setTypeaheadResults: function(results, flags = {}) {
        this.world().performAction({name: 'setTypeaheadResults',
            transaction: (worldState) =>
                worldState.setIn(
                    ['Geo', 'autocompleteResults', this._guid],
                    Immutable.fromJS(results))
                .setIn(
                    ['Geo', 'locationIsUnrecognized', this._guid],
                    !results.length && !flags.short)
        });
    },

    componentWillUnmount() {
        // store the current rootNodeId because after we defer the transaction,
        // the component will have already been unmounted, thus the rootNodeId
        // will be undefined.
        var previousRootNodeId = this._guid;
        this.world().performAction({name: 'clearLocationState',
            transaction: (ws) => ws.deleteIn(['Geo', 'autocompleteResults', previousRootNodeId])
                                   .deleteIn(['Geo', 'locationIsUnrecognized', previousRootNodeId])
        });
    },

    typeaheadOptions: function() {
        return this.world().getIn(
            ['Geo', 'autocompleteResults', this._guid],
            Immutable.List());
    },

    locationIsUnrecognized: function() {
        return this.world().getIn(
            ['Geo', 'locationIsUnrecognized', this._guid],
            false);
    },

    render: function () {
        let showFallbackPrompt = !!this.props.value.get('location') &&
            this.locationIsUnrecognized() &&
            this.props.allowFallback;
        return <div className={'field location-editor ' + this.props.className}
                    onKeyUp={this.props.onKeyUp}
                    onFocus={this.props.onFocus}
                    onBlur={this.props.onBlur}>
            {this.props.value.get('fallback')
                ? <LocationFallback
                    value={this.props.value}
                    validation={this.props.validation}
                    showErrors={this.props.showErrors}
                    onReset={this.fallbackWasReset}
                    onChange={(value) => {
                        this.props.onChange(value);
                        this.props.onSelect && this.props.onSelect(value);
                    }}
                    element={this.props.element} />
                : <div>
                    <LocationTypeahead
                        ref="typeahead"
                        {...this.props} // do not clobber the below value by moving this line
                        value={this.props.value.get('location') || ''}
                        onInput={this.setSearchInput}
                        onFocusOption={(value, element) => this.pickLocation(value)}
                        onSelect={this.handleTypeaheadSelect}
                        position={this.position}
                        options={this.typeaheadOptions()}
                    />
                {showFallbackPrompt &&
                    <FallbackPrompt ref="prompt" onYes={this.yesFallback} onNo={this.noFallback} />}
                </div>
            }
        </div>;
    },

    handleTypeaheadSelect(value) {
        this.pickLocation(value, this.props.onSelect);
    },

    onFocus: function(event) {
        // The relatedTarget is the element that is losing focus when we
        // receiving it or receiving it when we lose it.
        // NB: If there is no relatedTarget, we still forward the event.
        if (! this.getDOMNode().contains(event.relatedTarget)) {
            this.props.onFocus(event);
        }
    },

    onBlur: function(event) {
        if (! this.getDOMNode().contains(event.relatedTarget)) {
            this.props.onBlur(event);
        }
    },

    setSearchInput: function(location) {
        let callback = this.props.onSelect || this.props.onChange;
        callback(LocationField.blankLocation({location}));
        this.setState({currentSearchPrefix: location});
        if (location.length < 3) {
            // short circuit the async search process
            this.setTypeaheadResults([], {short: true});
        } else {
            let stamp = Date.now();
            this.fetchLocations(location,
                Object.assign({}, this.props.autocompleteEndpointOptions, {time: stamp}),
                (result, options) => {
                    if (options && options.time) {
                        this.lastArrivedRequest = Math.max(options.time, this.lastArrivedRequest);
                        if (options.time === this.lastArrivedRequest) {
                            this.setTypeaheadResults(result);
                        }
                    } else {
                        this.setTypeaheadResults(result);
                    }
                }
            );
        }
    },

    pickLocation: function(name, callback) {
        callback = callback || this.props.onChange;
        var loc = this.typeaheadOptions()
            .filter(o => o.get('matched_name') == name)
            .first();
        callback(LocationField.blankLocation(Immutable.Map({
            geoid: loc.get('geoid'),
            location: loc.get('matched_name'),
            formatted: loc.get('formatted_name'),
            countryCode: loc.get('country'),
            // FIXME: awb.geo is returning null for the regionCode of many
            // but not quite all place results. :(
            // see GEO-150
            regionCode: loc.get('code') || undefined,
            type: loc.get('type')
        })));
    },

    componentDidMount: function() {
        this._guid = cuid();
        this.lastArrivedRequest = Date.now();
    },

    componentWillUpdate: function() {
        this.position = this.getDOMNode().offsetTop;
    }
});

module.exports = LocationField;
