"use strict";

var React = require('react');
var Immutable = require('immutable');

var Auth = require('dealers/Auth');
var Signin = require('dealers/Signin');
var Valid = require('abc/validators');
var Wish = require('Wish');
var World = require('World');
var {I18N, i18n} = require('I18N');


var Engage = require('./JanrainEngage');
var FormInput = require('elements/FormInput');
var PaneNavButtons = require('elements/PaneFlow').PaneNavButtons;
var Spinner = require('elements/Spinner');

var ForgotPassword = require('./ForgotPassword');


var LoginPassword = React.createClass({
    mixins: [World.Child, Engage.Mixin],

    getInitialState () {
        return {
            showPasswordValidation: false
        };
    },

    componentDidUpdate: function() {
        if (!Auth.isUnauthed(this.world())) {
            this.props.onClose();
        }
    },

    login: function() {
        this.setState({showPasswordValidation: true});
        if (Valid.notEmpty(Signin.getField('password')).ok) {
            Auth.logInByEmail(this.world(),
                              Signin.getField('email'),
                              Signin.getField('password'));
        }
    },

    retry: function() {
        Auth.retryLogIn(this.world(), Signin.getField('email'), Signin.getField('password'));
    },

    render: function() {
        var email = Signin.getField('email');
        var password = Signin.getField('password');

        // attempt : maybe wish
        var attempt = Auth.loginAttemptWithEmailAndPassword(this.world(), email, password);
        var attemptStatus; // in {'Success', 'BadPassword', 'UnverifiedEmail', 'Pending', 'Error', 'NotAttempted'}
        var failureMessage; // comes from server

        if (attempt) {
            attemptStatus = Wish.look(attempt, {
                Available: (v) => {
                    let error = v.get('error');
                    if (error) {
                        if (v.getIn(['body','error']) === 'Pending verification.') {
                            return 'UnverifiedEmail';
                        } else {
                            return 'BadPassword';
                        }
                    }
                    return 'Success';
                },
                Pending: () => <Spinner/>,
                Error: () => 'Error'
            });
        } else {
            attemptStatus = 'NotAttempted';
        }

        if (attemptStatus === 'UnverifiedEmail') {
            setTimeout(() => this.props.onUnverified(), 0);
        }

        var attemptMessage = {
            'Success': i18n('You have logged in.'),
            'UnverifiedEmail': '',
            'BadPassword': i18n('That email or password is not correct.'),
            'Pending': <Spinner/>,
            'Error': i18n('Something went wrong. Please try again.'),
            'NotAttempted': ''
        }[attemptStatus];

        return this.props.inline ?
            this.renderInlineMarkup(email, password, attemptStatus, attemptMessage)
            :
            this.renderDefaultMarkup(email, password, attemptStatus, attemptMessage);
    },

    renderDefaultMarkup(email, password, attemptStatus, attemptMessage) {
        return <div className="flow-pane signup-flow">
            <h3><I18N>Log in</I18N></h3>
            <p><I18N>Welcome back! Please enter your password.</I18N></p>
            <div className="field">
                <label><I18N>Email address:</I18N></label>
                <FormInput type="email" name="email" ref="email"
                    disabled={true}
                    value={email}
                    onChange={Signin.changeHandlerFor('email')}
                    onKeyUp={this.keyAdvance}
                    validation={Valid.emailAddress(email)} />
            </div>
            <div className="field">
                <label><I18N>Password</I18N></label>
                <FormInput type="password" name="password" ref="password"
                    value={password}
                    onKeyUp={this.keyAdvance}
                    onChange={this.passwordChanged}
                    onFocus={() => this.setState({showPasswordValidation: false})}
                    validation={Valid.gated(this.state.showPasswordValidation,
                        this.passwordValidation(attemptStatus, attemptMessage))}
                    />
            </div>

            <ForgotPassword email={email} />
            {attemptStatus === 'Error' && <p style={{marginTop: '-2.5em'}}>{attemptMessage}</p>}
            <PaneNavButtons {...this.props}
                valid={true}
                atFlowEnd={false}
                nextButtonText={attemptStatus === 'Pending' ? '...' : i18n('Log in')}
                goNext={this.login} />
        </div>;
    },

    renderInlineMarkup(email, password, attemptStatus, attemptMessage) {
        return <div className="flow-pane signup-flow">
            <div className="create-account">
                <h3><I18N>Log in</I18N></h3>
                <p><I18N>Welcome back! Please enter your password.</I18N></p>
                <div className="field">
                    <label><I18N>Email address:</I18N></label>
                    <FormInput type="email" name="email" ref="email"
                        value={email}
                        onChange={Signin.changeHandlerFor('email')}
                        onKeyUp={this.keyAdvance}
                        validation={Valid.gated(this.state.showPasswordValidation,
                            Valid.emailAddress(email))} />
                </div>
                <div className="field">
                    <label><I18N>Password</I18N></label>
                    <FormInput type="password" name="password" ref="password"
                        value={password}
                        onKeyUp={this.keyAdvance}
                        onChange={this.passwordChanged}
                        onFocus={() => this.setState({showPasswordValidation: false})}
                        validation={Valid.gated(this.state.showPasswordValidation,
                            this.passwordValidation(attemptStatus, attemptMessage))}
                        />
                </div>

                <ForgotPassword email={email} />
                {attemptStatus === 'Error' && <p style={{marginTop: '-2.5em'}}>{attemptMessage}</p>}
                <PaneNavButtons {...this.props}
                    valid={true}
                    atFlowEnd={false}
                    nextButtonText={attemptStatus === 'Pending' ? '...' : i18n('Log in')}
                    prevButtonText={i18n('&lsaquo;&lsaquo; Back to sign up')}
                    goNext={this.login} />
            </div>
            <div className="social-log-in">
                <p><I18N><strong i18n-id="or">or</strong> log in with social media:</I18N></p>
                <Engage.Authenticator
                    onLogin={this.janrainLoginSucceeded}
                    onSignup={this.janrainSignupInitiated} />
            </div>
        </div>;
    },

    passwordChanged(evt) {
        Signin.changeHandlerFor('password')(evt);
        this.setState({showPasswordValidation: false});
    },

    passwordValidation(attemptStatus, errorMsg) {
        var ok = attemptStatus !== 'Failure' && attemptStatus !== 'BadPassword';
        return Valid.allValid([
            Valid.notEmpty(Signin.getField('password')),
            {ok, errorMsg}
        ]);
    },

    keyAdvance: function(evt) {
        if (evt.key === 'Enter') {
            evt.preventDefault();
            this.login();
        }
    },

    componentDidMount: function() {
        this.refs.password.focus();
    }
});


module.exports = LoginPassword;
