"use strict";

var React = require('react');
var Immutable = require('immutable');

var DOM = require('DOM');
var config = require('config');
var Spinner = require('elements/Spinner');


var cleanUpCaptcha = function() {
    var container = document.getElementsByClassName('pls-container');
    if (container.length) {
        for (var r = 0; r < container.length; r++) {
            container[r].parentNode.removeChild(container[r]);
        }
    }
    // <3 this profession: https://groups.google.com/forum/#!topic/recaptcha/miaW9qdVIgA
};


const PRELOAD = 0;
const LOADING = 1;
const LOADED = 2;

var captchaState = PRELOAD;
var _captchaLoadListeners = [];

global.captchaLoaded = function() {
    captchaState = LOADED;
    _captchaLoadListeners.forEach(fn => fn());
};

var CaptchaLoadListeners = {
    add(fn) {
        _captchaLoadListeners.indexOf(fn) < 0 && _captchaLoadListeners.push(fn);
    },
    remove(fn) {
        var index = _captchaLoadListeners.indexOf(fn);
        if (index >= 0) {
            _captchaLoadListeners.splice(index, 1);
        }
    }
};

var Preloader = React.createClass({
    statics: {
        CAPTCHA_URL: 'http://www.google.com/recaptcha/api.js?' +
                        'onload=captchaLoaded&render=explicit'
    },

    componentDidMount () {
        if (captchaState === PRELOAD) {
            captchaState = LOADING;
            var captchaBrain = document.createElement('SCRIPT');
            captchaBrain.setAttribute('src', Preloader.CAPTCHA_URL);
            captchaBrain.async = captchaBrain.defer = true;
            document.head.appendChild(captchaBrain);
        }
    },

    render () {
        return null;
    }
});


var NoCaptcha = React.createClass({

    propTypes: {
        onChange: React.PropTypes.func.isRequired,
        isMobile: React.PropTypes.bool
    },

    getDefaultProps() {
        return { isMobile: false };
    },

    componentWillMount() {
        CaptchaLoadListeners.add(this.renderCaptcha);
    },

    componentWillUnmount() {
        CaptchaLoadListeners.remove(this.renderCaptcha);
        cleanUpCaptcha();
    },

    componentDidMount() {
        this.renderCaptcha();
    },

    render() {
        var regularStyle = {
            minHeight: '7.7rem'
        };
        var mobileStyle = {
            webkitTransform:'scale(0.85)',
            transform:'scale(0.85)',
            webkitTransformOrigin:'0 0',
            transformOrigin:'0 0'
        };
        return <div style={this.props.isMobile
                ? Object.assign({}, mobileStyle, regularStyle)
                : regularStyle}
            className="reCaptcha">
            <div ref="destination"/>
            {captchaState !== LOADED ? <Spinner /> : null}
        </div>;
    },

    // since we don't have a way to dangerouslySetInnerHTML here, do not touch
    shouldComponentUpdate() {
        return false;
    },

    renderCaptcha() {
        if (captchaState === LOADED) {
            this.forceUpdate(); // JUST ONCE, override shouldComponentUpdate being false to hide the spinner
            grecaptcha.render(React.findDOMNode(this.refs.destination), {
                sitekey: config.idealist['google.recaptcha.v2.public_key'],
                callback: this.onResponseSuccess.bind(this)
            });
        }
    },

    onResponseSuccess(resp) {
        window.scrollTo(0, DOM.offsetTop(React.findDOMNode(this)) - 60);
        this.props.onChange(Immutable.fromJS({response: resp}));
    }
});

module.exports = {Preloader, NoCaptcha};
