"use strict";

var React = require('react/addons');
var _ = require('underscore');

var {i18n} = require('I18N');
var Signin = require('dealers/Signin');
var Valid = require('abc/validators');
var World = require('World');

var PaneFlow = require('elements/PaneFlow');
var PaneProps = require('elements/PaneFlow/PaneProps');
var ValidationHelper = require('elements/ValidationHelper');

var NameLocView = require('./NameLocView');
var NameLocInlineView = require('./NameLocInlineView');
var NameLocJanrainView = NameLocView;
var NameLocInlineJanrainView = require('./NameLocInlineJanrainView');


var SignupNameLoc = React.createClass({
    mixins: [World.Child, ValidationHelper],

    propTypes: PaneProps,

    // For ValidationHelper
    validators: {
        email: () => Valid.emailAddress(Signin.getField('email')),
        full_name: () => Valid.notEmpty(Signin.getField('full_name')),
        first_name: () => Valid.lengthInRange(Signin.getField('first_name'), 1, 128),
        location: () => Valid.location(Signin.getField('location'))
    },

    render: function() {
        var full_name = Signin.getField('full_name');
        var first_name = Signin.getField('first_name');
        var location = Signin.getField('location');
        var paneStyles = this.props.inline ? {} : {minHeight: 450};

        var isJanrain = this.props.isJanrainFlow ? 1 : 0;
        var isInline = this.props.inline ? 1 : 0;
        var viewMatrix = [[NameLocView, NameLocInlineView],
            [NameLocJanrainView, NameLocInlineJanrainView]];
        var viewComponent = viewMatrix[isJanrain][isInline];
        // I KNOW KUNG FU. anyway, a diagram:
        //  *-----------------*------------------*
        //  | modal, password | inline, password |
        //  *-----------------+------------------*
        //  | modal, janrain  | inline, janrain  |
        //  *-----------------*------------------*

        var goNext = this.props.goNext;
        if (this.props.inline) {
            goNext = this.props.attemptSignup;
        }

        return <div className="flow-pane signup-flow" style={paneStyles}>
            {React.createElement(viewComponent, _.extend({
                ref: 'view',
                email: Signin.getField('email'),
                emailValidation: this.showValid('email'),
                fullName: full_name,
                fullNameValidation: this.showValid('full_name'),
                firstName: first_name,
                firstNameValidation: this.showValid('first_name'),
                location: location,
                locationValidation: this.showValid('location'),
                hideFieldValidation: this.hideFieldValidation,
                showFieldValidation: this.showFieldValidation,
                changeLocation: this.changeLocation,
                isValid: this.isValid(),
                keyAdvance: this.keyAdvance
            }, this.props))}
            <PaneFlow.PaneNavButtons {...this.props}
                valid={this.isValid()}
                onInvalid={this.onInvalid}
                goNext={goNext}
                nextButtonText={this.props.inline && i18n('Sign up')} />
        </div>;
    },

    isValid: function () {
        var conditions = _.map(this.validators, (func) => func());
        if (!!this.refs.view) {
            return Valid.allValid(conditions).ok && this.refs.view.isValid();
        }
        return Valid.allValid(conditions).ok;
    },

    onInvalid: function () {
        this.showAllFieldValidation();
        // cover any extras in individual views
        if (this.refs.view) {
            this.refs.view.onInvalid && this.refs.view.onInvalid();
        }
    },

    // handled separately because it isn't called with an event
    changeLocation: function(val) {
        Signin.setField('location', val);
    },

    keyAdvance: function(event) {
        if (event.key === 'Enter' && !this.props.inline) {
            event.preventDefault();
            if (this.isValid()) {
                this.props.goNext();
            } else {
                this.onInvalid();
            }
        }
    }
});


module.exports = SignupNameLoc;
