"use strict";

var React = require('react/addons');

var Router = require('Router');
var Valid = require('abc/validators');
var {I18N, i18n} = require('I18N');

var Client = require('dealers/Client');
var Signin = require('dealers/Signin');
var World = require('World');

var Captcha = require('elements/LoginSignup/NoCaptcha');
var FormInput = require('elements/FormInput');
var Look = require('elements/Look');
var PaneFlow = require('elements/PaneFlow');
var Spinner = require('elements/Spinner');

var FieldValidationHelper = require('./FieldValidationHelper');


var SignupSecure = React.createClass({
    mixins: [FieldValidationHelper, World.Child],

    fields: ['Password'],

    render: function() {
        var password = Signin.getField('password');

        return this.props.inline
            ? this.renderFields(password)
            : <div className="flow-pane signup-flow">
                <h1><I18N>Secure your account</I18N></h1>
                {this.renderFields(password)}
                <PaneFlow.PaneNavButtons {...this.props}
                    valid={this.isValid()}
                    onInvalid={this.onInvalid}
                    atFlowEnd={false}
                    goNext={this.props.attemptSignup}
                    nextButtonText={i18n('Sign up')} />
            </div>;
    },

    renderFields(password) {
        return <div>
            <div className="field">
                <label><I18N>Password</I18N></label>
                <FormInput type="password" name="password"
                    placeholder={i18n('Passwords must be at least 6 characters')}
                    ref="password"
                    value={password}
                    onChange={Signin.changeHandlerFor('password')}
                    onFocus={this.hideFieldValidation('Password')}
                    validation={Valid.gated(this.state.showPasswordValidation,
                        Valid.longerThan(password, 5))} />
            </div>
            <div className="field">
                <span><I18N>Please verify that you're a human:</I18N></span>
                <p><Captcha.NoCaptcha
                    isMobile={Client.isMobile(this.world(), 320)}
                    onChange={Signin.changeHandlerFor('captcha')} /></p>
            </div>
            <p><small>
                <I18N>By clicking "Sign up" you're agreeing to our <a i18n-id="terms" target="_blank"
                    href={Router.urlForRoute(Router.route('info/view-path', {path: 'About/Terms'}))}>
                Terms & Conditions</a></I18N>
            </small></p>

            {this.props.signupWish && <Look wish={this.props.signupWish}>
                {{Pending: () => <Spinner/>,
                  Available: () => null,
                  Error: () => <I18N>Something went wrong, please try again.</I18N>}}
            </Look>}
        </div>;
    },

    validators() {
        return [
            Valid.longerThan(Signin.getField('password'), 5),
            Valid.passedCaptcha(Signin.getField('captcha'))
        ];
    },

    isValid() {
        return Valid.allValid(this.validators()).ok;
    },

    onInvalid() {
        this.setState({showPasswordValidation: true});
    },

    componentDidMount: function() {
        if (!this.props.inline) {
            this.refs.password.focus();
        }
    }
});


module.exports = SignupSecure;
