"use strict";

var Immutable = require('immutable');
var React = require('react/addons');

var Auth = require('dealers/Auth');
var Signin = require('dealers/Signin');
var Valid = require('abc/validators');
var Wish = require('Wish');
var World = require('World');

var FormInput = require('elements/FormInput');
var If = require('abc/if');
var LocationField = require('elements/LocationField');
var PaneNavButtons = require('elements/PaneFlow').PaneNavButtons;
var PaneProps = require('elements/PaneFlow/PaneProps');

var Engage = require('./JanrainEngage');
var FieldValidationHelper = require('./FieldValidationHelper');

var {I18N, i18n} = require('I18N');


var SignupStartPane = React.createClass({
    mixins: [World.Child, FieldValidationHelper, Engage.Mixin],
    propTypes: PaneProps,

    getDefaultProps: function() {
        return {
            // TODO: check the config for janrain
            janrain: true,
            inline: false
        };
    },

    getInitialState: function() {
        return {
            showValidation: false
        };
    },

    canAdvanceEmail: function() {
        if (this.isValid()) {
            return Wish.look(Auth.userExistsWithEmail(this.world(), Signin.getField('email')),
                {Pending: ()=> false,
                 Available: ()=> true,
                 Error: ()=> false});
        } else {
            return false;
        }
    },

    componentDidUpdate() {
        if (this.state.triedToAdvance) {
            Wish.look(Auth.userExistsWithEmail(this.world(), Signin.getField('email')), {
                Pending: () => {},
                Error: () => this.setState({triedToAdvance: false}),
                Available: () => this.checkEmailAndProceed()
            });
        }
    },

    checkEmailAndProceed: function() {
        if (this.isValid()) {
            Wish.look(Auth.userExistsWithEmail(this.world(), Signin.getField('email')), {
                Pending: () => this.setState({triedToAdvance: true}),
                Error: () => null,
                Available: exists => {
                    this.props.switchToLoginFlow(exists);
                    this.props.goNext();
                }
            });
        }
        else {
            this.setState({showValidation: true, triedToAdvance: false});
        }
    },

    render: function() {
        return this.props.inline ? this.renderInlineMarkup() : this.renderDefaultMarkup();
    },

    renderDefaultMarkup: function() {
        var email = Signin.getField('email');
        return <div className="flow-pane signup-flow">
            <div className="sign-in">
                <h1><I18N>Sign in with:</I18N></h1>
                <Engage.Authenticator
                    onLogin={this.janrainLoginSucceeded}
                    onSignup={this.janrainSignupInitiated} />
                <div className="field">
                    <label><I18N>Or, use your email address:</I18N></label>
                    <FormInput type="email" name="email"
                        ref="emailAddress"
                        value={email}
                        onChange={Signin.changeHandlerFor('email')}
                        onKeyUp={this.keyAdvance}
                        onFocus={() => this.setState({showValidation: false})}
                        validation={Valid.gated(this.state.showValidation,
                                                Valid.emailAddress(email))} />
                </div>
            </div>
            <PaneNavButtons {... this.props}
                onInvalid={() => this.setState({showValidation: true})}
                valid={this.canAdvanceEmail()}
                goNext={this.checkEmailAndProceed} />
        </div>;
    },

    renderInlineMarkup: function() {
        var email = Signin.getField('email');
        return <div className="flow-pane signup-flow">
            <div className="create-account">
                <div className="email-signin">
                    <label><I18N>Enter your email address:</I18N></label>
                    <FormInput type="email" name="email"
                        ref="emailAddress"
                        value={email}
                        onChange={Signin.changeHandlerFor('email')}
                        onKeyUp={this.keyAdvance}
                        onFocus={() => this.setState({showValidation: false})}
                        validation={Valid.gated(this.state.showValidation,
                                                Valid.emailAddress(email))} />
                </div>
                <PaneNavButtons {... this.props}
                    onInvalid={() => this.setState({showValidation: true})}
                    valid={this.canAdvanceEmail()}
                    goNext={this.checkEmailAndProceed} />
            </div>
            <div className="social-log-in">
                <span><I18N>or use social media:</I18N></span>
                <Engage.Authenticator
                    onLogin={this.janrainLoginSucceeded}
                    onSignup={this.janrainSignupInitiated} />
                <p className="log-in">
                    <I18N>Already have an Idealist account?&nbsp;
                        <a i18n-id="login" href="#" ref="loginLink" onClick={this.forceSwitchToLogin}>
                            Log in
                        </a>&nbsp;
                    to get started.</I18N>
                </p>
            </div>
        </div>;
    },

    keyAdvance: function(evt) {
        if (evt.key === 'Enter') {
            evt.preventDefault();
            this.checkEmailAndProceed();
        }
    },

    isValid: function() {
        return Valid.emailAddress(Signin.getField('email')).ok;
    },

    focus: function() {
        if (!this.props.inline) {
            this.refs.emailAddress.focus();
        }
    },

    forceSwitchToLogin: function() {
        this.props.switchToLoginFlow(true);
        this.props.goNext();
    }
});


module.exports = SignupStartPane;
