'use strict';

var React = require('react/addons');
var {i18n} = require('I18N');

var World = require('World');
var API = require('API');
const config = require('config');

var OrgIntakeForm = require('dealers/OrgIntakeForm');

var ImageUploader = require('abc/image_uploader');
var Multiselect = require('abc/forms/multiselect');
var RTE = require('abc/rte');

var Look = require('elements/Look');
var OrgField = require('elements/OrgIntake/OrgField');
var Spinner = require('elements/Spinner');
var ValidationHelper = require('elements/ValidationHelper');
var Validation = require('elements/Validation');

var Org = require('schemas/Org');


var KeywordsInput = React.createClass({
    mixins: [World.Child],

    render: function() {
        var {value, ...other} = this.props;

        return <Look.Quick wish={this.world().getIn(['Constants', 'areaOfFocus'])}>
            {keywords =>
                <Multiselect {...other}
                    selectedChoices={value.toJS()}
                    choices={keywords.toJS()}
                />}
        </Look.Quick>;
    }
});

var UploadcareInput = React.createClass({
    render() {
        return <ImageUploader
            name={this.props.name}
            existingImage={this.props.value}
            onChange={this.props.onChange} />;
    }
});

var RTEInput = React.createClass({
    mixins: [World.Child],

    getValue() {
        return this._rte.getValue();
    },

    render() {
        // TODO: configuration & stylesheet url
        return <Look.Quick wish={this.world().getIn(['Constants', 'restrictiveRTE'])}>
            {rteConfig =>
                <RTE value={this.props.value}
                    ref={element => this._rte = element}
                    onChange={this.props.onChange}
                    wysihtml5Configuration={rteConfig.toJS()}
                    editorStylesheetUrl={'/@/' + config.idealist.release + '/blue-1/css/rich-text-editor.css'}
                />}
        </Look.Quick>;
    }
});

module.exports = React.createClass({
    displayName: 'NewOrgCreationForm',

    mixins: [World.Child],

    getInitialState() {
        return {
            hideValidation: true,
            invalid: null
        };
    },

    getRTEValue() {
        return this.refs.rte.getValue();
    },

    nameAndPropsFor(name) {
        return {name, showErrors: this.props.showErrors};
    },

    render() {
        return <fieldset>
            <OrgField {...this.nameAndPropsFor('shortName')}
                autoFocus={true}
                label={i18n('Organization nickname')}
                help={i18n('This can be an acronym, abbreviation, or the first word of your organization\'s name (up to 25 characters).')}
            />
            <OrgField {...this.nameAndPropsFor('streetAddress')}
                label={i18n('Street address')}
                placeholder={i18n('e.g. 123 Main St.')} />
            <OrgField {...this.nameAndPropsFor('deliveryDetails')}
                label={i18n('Street address 2')}
                placeholder={i18n('e.g. 11th Floor')} />
            <OrgField {...this.nameAndPropsFor('ein')}
                label={i18n('Government ID number')}
                help={i18n('For example, in the U.S., this would be your Employer Identification Number (EIN).')} />
            <OrgField {...this.nameAndPropsFor('website')}
                label={i18n('Organization website')} />
            <OrgField {...this.nameAndPropsFor('description')}
                externalWidget={true}
                ref='rte'
                type={RTEInput}
                label={i18n('Describe your organization\'s mission')} />
            <OrgField {...this.nameAndPropsFor('keywords')}
                externalWidget={true}
                required={true}
                type={KeywordsInput}
                label={i18n('Areas of focus')} />
            <OrgField {...this.nameAndPropsFor('imageUrl')}
                type={UploadcareInput}
                label={i18n('Upload a photo or logo')} />
        </fieldset>;
    }
});
