var React = require('react');
import {i18n, I18N} from 'I18N';
import {Validation} from 'conformist';

var Field = require('elements/Forms/Field');
var InputWrapper = require('elements/Forms/InputWrapper');


var OrgValidationMessage = React.createClass({
    render() {
        // For mocking.
        var TransitionGroup = OrgValidationMessage.TransitionGroup;

        return <div className={['error-slot', this.props.externalWidget ? 'external-widget-cushion' : null].join(' ')}>
            <TransitionGroup transitionName="input-error" transitionAppear={true}>
                {this.props.messages && !!this.props.messages.length &&
                    <div key="error" className="error-slider">
                        <div className="error-balloon">{this.props.messages.map(m => <div key={m}>{m}</div>)}</div>
                    </div>}
            </TransitionGroup>
        </div>;
    }
});


var OrgField = React.createClass({
    mixins: [Field],

    propTypes: {
        label: React.PropTypes.string,
        help: React.PropTypes.string,
        type: React.PropTypes.oneOfType([
            React.PropTypes.func,
            React.PropTypes.string
        ]),
        className: React.PropTypes.string,
        disabled: React.PropTypes.bool,
        showErrors: React.PropTypes.bool
    },

    getValue() {
        var input = this.refs.input;
        return input.getValue ? input.getValue() : input.value;
    },

    render() {
        var element = this.schemaElement();
        var errors = element.errors;
        var {type, label, help, className, required, showErrors, ...props} = this.props;
        var Input;

        if (typeof type === 'function') {
            Input = type;
        } else {
            Input = InputWrapper;
        }

        return <div style={{position: 'relative'}} className={this.props.className || ''}>
            <p className="hint">{this.props.help}</p>
            <label>
                {label || props.name}
                {(required || element.hasValidator(Validation.Value.Present))
                    && <I18N><span i18n-id="required" className="required">(Required)</span></I18N>}
            </label>
            <Input type={type}
                ref='input'
                onChange={this.onChange}
                value={element.value}
                showErrors={showErrors}
                {...props}
            />
            {showErrors && <OrgValidationMessage
                externalWidget={props.externalWidget}
                messages={element.errors} />}
        </div>;
    }
});

// Expose for testing with mocks.
OrgValidationMessage.TransitionGroup = React.addons.CSSTransitionGroup;
OrgField.OrgValidationMessage = OrgValidationMessage;

module.exports = OrgField;
