// The Org schema expects addresses (1 or more), but LocationField generates
// only one. This wrapper is given an array of Location
// schema instances, but uses only the first in LocationField.
//
// ALSO: we control the value for the LocationField with a state variable,
// and only propagate changes when the user has seleted a typeahead result,
// OR the user has changed a fallback field.

"use strict";

var React = require('react');
var LocationField = require('elements/LocationField');
var IDOPropTypes = require('IDOPropTypes');
var OrgIntakeForm = require('dealers/OrgIntakeForm');
var Immutable = require('immutable');
var {Map, List} = Immutable;

var OrgLocationInput = React.createClass({
    propTypes: {
        value: React.PropTypes.oneOfType([
            React.PropTypes.array,
            IDOPropTypes.immutable.map
        ]),
        element: IDOPropTypes.conformist,
        onChange: React.PropTypes.func.isRequired,
        showErrors: React.PropTypes.bool,
        placeholder: React.PropTypes.string
    },

    getInitialState() {
        return { value: this.props.value };
    },

    chooseLocation(location) {
        OrgIntakeForm.setField(['addresses', 0], location);
        this.props.onChange(location);
    },

    setValue(value) {
        this.setState({value});
    },

    setValueFromConformist(isValid, elm) {
        this.setState({value: elm.value});
    },

    componentWillMount() {
        this.props.element.observe(this.setValueFromConformist);
    },

    componentWillUnmount() {
        this.props.element.unobserve(this.setValueFromConformist);
    },

    render() {
        return <LocationField
            element={this.props.element}
            onSelect={this.chooseLocation}
            onChange={this.setValue}
            onBlur={evt => {
                if (evt) {
                    if (!evt.relatedTarget) {
                        this.chooseLocation(this.state.value);
                    }
                }
            }}
            showErrors={this.props.showErrors}
            value={this.state.value}
            maxMenuHeight={350}
	    autocompleteEndpointOptions={{type: 'place'}}
	    placeholder={this.props.placeholder}/>;
    }
});

module.exports = OrgLocationInput;
