"use strict";

var React = require('react');
var _ = require('underscore');

var DOM = require('DOM');
var {I18N, i18n} = require('I18N');
var World = require('World');

var Analytics = require('dealers/AnalyticsDealer');
var OrgIntakeForm = require('dealers/OrgIntakeForm');

var Form = require('elements/Forms/Form');
var OrgField = require('elements/OrgIntake/OrgField');
var NewOrgCreationForm = require('elements/OrgIntake/NewOrgCreationForm');
var OrgNameDeDuper = require('elements/OrgIntake/OrgNameDeDuper');
var Spinner = require('elements/Spinner');

var OrgTypeInput = require('./OrgTypeInput');
var OrgLocationInput = require('./OrgLocationInput');
var ScriptPreLoader = require('./ScriptPreLoader');


var OrgIntakeCore = React.createClass({
    mixins: [World.Child],

    childContextTypes: {
        schema: React.PropTypes.object,
        onChange: React.PropTypes.func.isRequired
    },

    getChildContext() {
        return {
            schema: OrgIntakeForm.state,
            onChange: this.handleChange
        };
    },

    propTypes: {
        authed: React.PropTypes.bool.isRequired
    },

    getInitialState() {
        return {
            allowsEditing: true,
            showErrors: false
        };
    },

    componentDidMount() {
        OrgIntakeForm.loadFromLocalStorage();
        this.forceUpdate();
    },

    handleChange(success, changedField) {
        changedField.validate();
        if (success && (changedField.name === 'type' ||
                        changedField.parent.name === 'addresses' ||
                        changedField === OrgIntakeForm.state)) {
            OrgIntakeForm.updateLocalStorage();
        }
        if (this.state.showErrors && !this.completeFormVisible() && this.validateHeader()) {
            this.setState({ showErrors: false });
        }
    },

    render() {
        let address = OrgIntakeForm.getMember(['addresses', 0]);
        let addressValid = address ? address.valid : false;
        let disableOrgName = !addressValid;
        let showAddButton = this.props.authed && !OrgIntakeForm.existingOrgIsSelected();
        let isSubmitting = OrgIntakeForm.isSubmitting(this.world());
        let dimAddButton = !this.completeFormVisible() || isSubmitting;
        return <Form
            className="create-org"
            schema={OrgIntakeForm.state}
            onChange={this.handleChange}
            onSubmit={this.submit}
        >
            <div>
                <OrgField name="type"
                    type={OrgTypeInput}
                    label={i18n('What kind of organization do you represent?')}
                    required={true} />
            </div>

            <fieldset>
                <OrgField name="addresses"
                    element={address}
                    type={OrgLocationInput}
                    required={true}
                    placeholder={i18n("Enter your city")}
                    showErrors={this.state.showErrors}
                    label={i18n('Where is your org located?')} />
            </fieldset>

            {!this.props.authed &&
                <a className='unauth-upsell' href="#" onClick={this.jumpToLogin}>
                    <I18N>Log in or create your personal account &rsaquo;&rsaquo;</I18N>
                </a>}

            {this.props.authed &&
              <div>
                  {!OrgIntakeForm.existingOrgIsSelected()
                   ? <OrgField name="fullName"
                      type={OrgNameDeDuper}
                      label={i18n("What's the name of your organization?")}
                      disabled={disableOrgName}
                      showErrors={this.state.showErrors} />
                  : <OrgNameDeDuper onChange={v => OrgIntakeForm.setField('fullName', v)} />}
              </div>}

            <ScriptPreLoader showSpinner={this.completeFormVisible()}>
                {this.completeFormVisible()
                    ? <NewOrgCreationForm
                        ref='newOrgForm'
                        showErrors={this.state.showErrors}/>
                    : null}
            </ScriptPreLoader>

            {!OrgIntakeForm.state.valid && this.state.showErrors && this.completeFormVisible() && <span className="server-error">
                {OrgIntakeForm.state.errors[0]}
            </span>}

            {showAddButton && <button type="button"
                // this key is to fool chrome into thinking the button is different
                // when we click it to get out of the deduper, thus preventing a
                // spurious scroll to the bottom of the page
                key={'addBtn' + this.completeFormVisible()}
                ref='addBtn'
                disabled={isSubmitting}
                onClick={this.submit}
                {...(dimAddButton ? {className: 'disabled'} : {})}
            >
                <I18N>Add</I18N>
            </button>}
            {isSubmitting && <Spinner style={{marginTop: '1em', marginRight: '-1em', float: 'right'}}/>}
        </Form>;
    },

    submit(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        if (this.completeFormVisible()) {
            this.refs.newOrgForm && OrgIntakeForm.setField(
                'description',
                this.refs.newOrgForm.getRTEValue()
            );

            // Always show errors on org submission b/c server may invalidate.
            this.setState({ showErrors: true });

            OrgIntakeForm.v2SubmitOrg(this.world());
        } else {
            // Turn off errors in complete form if Org header was valid.
            this.setState({ showErrors: !this.validateHeader() });
        }
    },

    validateHeader() {
        var fullNameValid = OrgIntakeForm.state.members.fullName.validate();
        var addressesValid = OrgIntakeForm.state.members.addresses.validate();
        return fullNameValid && addressesValid;
    },

    jumpToLogin(evt) {
        evt.preventDefault();
        let emailField = document.querySelector('.user-intake');
        DOM.scrollToReveal(emailField);
        emailField.focus();
    },

    // Check whether the *New Org* form is visible
    completeFormVisible() {
        var org = OrgIntakeForm.selectedOrgItem();
        var visible = org && org.isAddNew && !org.isExact;
        if (visible && !this._hasTriggeredFormShownInAnalytics) {
            Analytics.trackPageview('/virtual/add-org/form-shown');
            this._hasTriggeredFormShownInAnalytics = true;
        }
        return visible;
    },

    shouldDisableOrgName() {
        return !this.locationValid() && _.isEmpty(OrgIntakeForm.getField('name'));
    }
});


module.exports = OrgIntakeCore;
