'use strict';

var React = require('react');
var _ = require('underscore');
var Immutable = require('immutable');
var Autocomplete = require('elements/Autocomplete');

var {I18N} = require('I18N');
var World = require('World');

var OrgIntakeForm = require('dealers/OrgIntakeForm');


var OrgNameDeDuper = React.createClass({
    mixins: [World.Child],

    render: function() {
        return OrgIntakeForm.existingOrgIsSelected() ?
            <DeduperExistingOrg
                org={OrgIntakeForm.selectedOrgItem()}
                onReset={(evt) => {
                    evt.preventDefault();
                    OrgIntakeForm.existingOrgWasReset(this.world());
                    this.props.onChange('');
                }} />
            :
            <DeduperTypeahead
                ref="typeahead"
                disabled={this.props.disabled}
                value={this.props.value}
                onChange={this.props.onChange} />;
    }
});

var DeduperTypeahead = React.createClass({
    mixins: [World.Child],

    getInitialState() {
        return { searchTerm: this.props.value };
    },

    render() {
        var typeaheadOpts = OrgIntakeForm.getDedupeResults(this.world(), this.state.searchTerm);

        var dedupeOptions = typeaheadOpts && typeaheadOpts.map(
            row => {return <Autocomplete.Option key={row.get('url') + row.get('name')}
                        value={row.toJS()}
                        label={row.get('name')}
                        allowAppendWhileFocused={row.get('isAddNew')}>
                {row.get('isAddNew') ?
                    <a href="#!" className="deduper-add-new-org">
                        <span className="deduper-add-label"><I18N>Add:</I18N>&nbsp;</span>
                        <span className="deduper-org-name">{row.get('name')}</span>
                    </a>
                    :
                    <a href="#!">
                        <span className="deduper-org-name">{row.get('name')}</span>
                        <span className="deduper-org-location">
                            {row.get('relativeLocationString')}
                        </span>
                    </a>}
            </Autocomplete.Option>;}
        ).toJS();

        return <fieldset className="deduper-container">
            <Autocomplete.Combobox
                ref='typeahead'
                value={this.props.value}
                valueComparator={(propVal, fieldVal) => {
                    return propVal && propVal.name === fieldVal;
                }}
                autocomplete='list'
                disabled={this.props.disabled}
                tabIndex={this.props.tabIndex}
                onInput={this.setSearchInput}
                onSelect={this.deduperOptionChosen}
                onBlur={this.deduperBlurred}
                maxMenuHeight={320}
                >
                {dedupeOptions ? dedupeOptions : []}
            </Autocomplete.Combobox>
        </fieldset>;
    },

    setSearchInput(searchTerm) {
        this.setState({ searchTerm: searchTerm });
        this.props.onChange(searchTerm);
        OrgIntakeForm.dedupeResultsFor(this.world(), searchTerm);
    },

    typeaheadWasChanged(deduperValue) {
        this.props.onChange(deduperValue.name);
    },

    deduperOptionChosen(deduperValue) {
        OrgIntakeForm.selectOrg(deduperValue);
    },

    deduperBlurred() {
        if (this.props.value && !OrgIntakeForm.selectedOrgItem()) {
            OrgIntakeForm.selectCurrentName();
        }
    },

    componentWillMount() {
        OrgIntakeForm.emptyDeduperResults(this.world());
    },

    componentDidMount() {
        if (OrgIntakeForm._orgIsResetting) {
            React.findDOMNode(this.refs.typeahead._input).focus();
            OrgIntakeForm._orgIsResetting = false;
        }
    }
});

var DeduperExistingOrg = React.createClass({
    render() {
        var org = this.props.org;
        return <div>
            <h2 className="existing-org-label">
                <I18N>This organization already exists!</I18N>
            </h2>
            <div className="existing-org">
                <img height="60" width="60" alt={org.name}
                    src={org.thumbUrl} />
                <span className="name">{org.name}</span>
                <span className="location">{org.relativeLocationString}</span>
                <ul className="actions">
                    <li><a href={org.url}>
                        <I18N>View org page</I18N>
                    </a></li>
                    <li><a href={org.becomeAdminUrl}>
                        <I18N>Become an admin</I18N>
                    </a></li>
                <li><a href="#" onClick={this.props.onReset}>
                        <I18N>Not it? Try again</I18N>
                    </a></li>
                </ul>
            </div>
        </div>;
    }
});

module.exports = OrgNameDeDuper;
