var _ = require('underscore');
var classNames = require('classnames');
var React = require('react');
var TransitionEvents = require('react/lib/ReactTransitionEvents');

var App = require('dealers/App');
var NavMenu = require('dealers/NavMenu');

var ClippedNumber = require('elements/ClippedNumber');
var Link = require('elements/Link');
var Look = require('elements/Look');
var Wish = require('Wish');
var World = require('World');
var {I18N, i18n} = require('I18N');

var AdministeredAssets = require('./AdministeredAssets');


var MobileNavMenu = React.createClass({
    mixins: [World.Child],

    getInitialState() {
        return {opened: false};
    },

    updateOpened(event) {
        if (event.animationName && event.target === this.getDOMNode()) {
            if (this.isOpen()) {
                NavMenu.toggleApplicationSlide(this.world(), true);
            } else {
                NavMenu.toggleApplicationSlide(this.world(), false);
            }
        }
    },

    isOpen() {
        return NavMenu.isMobileNavMenuOpen(this.world());
    },

    // see https://github.com/facebook/react/issues/1196
    componentDidMount() {
        TransitionEvents.addEndEventListener(this.getDOMNode(), this.updateOpened);
    },

    componentWillUnmount() {
        TransitionEvents.removeEndEventListener(this.getDOMNode(), this.updateOpened);
    },

    renderMenuTop(user) {
        var userInboxCount = user.get('inbox_count');

        return <ul className="user-navlinks">
            <li className="profile-link">
                <Link endpoint='me/view'>
                    <figure className="avatar-icon">
                         <img src={user.get('profile_img_url') || '/images/user/noPhoto-t.gif'}
                              alt={user.get('full_name')}
                              className="my-avatar" />
                          <figcaption className="profile-link"><I18N>Profile</I18N></figcaption>
                    </figure>
                </Link>
            </li>

            <li className="my-inbox-link">
                <Link endpoint='me/messages'>
                    <I18N>Inbox ({userInboxCount})</I18N>
                </Link>
            </li>

            <li className="my-dashboard-link">
                <Link endpoint='me/dashboard'><I18N>Dashboard</I18N></Link>
            </li>
        </ul>;
    },

    render() {
        var classes = classNames(
            _.extend(NavMenu.slidingClassNames(this.world(), true), {'mobile-user-nav': true}));

        var currentUser = App.currentUser(this.world());
        var administeredAssets = App.currentAdministeredAssets(this.world());

        return <nav id="mobile-user-menu" className={classes}>
            <Look.Quick wish={currentUser}>
                {(user) => this.renderMenuTop(user)}
            </Look.Quick>{/* TODO: let the user know there was an error */}
            <Look.Quick wish={administeredAssets}>
                {(assets) => <AdministeredAssets assets={assets} />}
            </Look.Quick>{/* TODO: let the user know there was an error */}
            <Look.Quick wish={currentUser}>
                {(user) => <span className="log-out">
                    <a href="/logout" className="logout"><I18N>Log out</I18N></a>
                </span>}
            </Look.Quick>
        </nav>;
    }
});

module.exports = MobileNavMenu;
