"use strict";

var React = require('react/addons');
var _ = require('underscore');
var classNames = require('classnames');

var LoginSignup = require('dealers/LoginSignup');
var NavMenu = require('dealers/NavMenu');
var World = require('World');
var {I18N, i18n} = require('I18N');

var Feature = require('elements/Feature');
var Link = require('elements/Link');
var SignupFlow = require('elements/LoginSignup/SignupFlow');
var PageHeaderAssetSearches = require('./PageHeaderAssetSearches');
var Search = require('./PageHeaderSearch');


var PageFooter = React.createClass({
    mixins: [World.Child],

    helpPath: function () {
        return 'Help'; // FIXME i18n
    },

    render: function () {
        var currentYear = new Date().getFullYear();
        return <footer id="footer" className={classNames("ido-footer", NavMenu.slidingClassNames(this.world()))} role="contentinfo">
            <div className="search-n-assets">
               <Search.PageFooterSearch />

               <div className="brand">
                  <div className="verticalize">
                    <img src="/images/logos/idealist_120.png" width="60" alt="Idealist.org" />
                  </div>
               </div>

               <h2 className="tagline"><I18N>We connect idealists with opportunities for action.</I18N></h2>
               <PageHeaderAssetSearches />
            </div>

            {LoginSignup.isFooterSignupOpen(this.world())
                ? <SignupFlow
                    className="susify"
                    onClose={() => LoginSignup.closeSignup(this.world())} />
                : null}

            <div className="susify">
                {/* Brand */}
                <div className="brand">
                    <div className="verticalize">
                        <img src="http://www.idealist.org/images/logos/idealist_120.png" width="60" alt="FIXME ALT" />
                    </div>
                </div>

                {/* Social */}
                <div className="social-links">
                    <div className="verticalize">
                        <ul>
                            <li>
                                <a id="linkedin" href="http://www.linkedin.com/groups?gid=1138797"><i className="social linkedin"></i><I18N>linkedin</I18N></a>
                            </li>
                            <li>
                                <a id="twitter" href="http://twitter.com/idealist"><i className="social twitter"></i><I18N>twitter</I18N></a>
                            </li>
                            <li>
                                <a id="facebook" href="http://www.facebook.com/Idealist"><i className="social facebook"></i><I18N>facebook</I18N></a>
                            </li>
                            <li className="youtube">
                                <a id="youtube" href="http://www.youtube.com/idealist"><i className="social youtube"></i><I18N>youtube</I18N></a>
                            </li>
                        </ul>
                    </div>
                </div>

                {/* Links */}
                <div className="links-n-legal">
                    <div className="verticalize">
                        <div className="stack">
                            <nav className="footer-links">
                                <ul>
                                    <li>
                                        <Link endpoint='about-us'><I18N>About</I18N></Link>
                                    </li>
                                    <li>
                                        <Link endpoint='info/view-path' values={{path: 'About/Vision'}}><I18N>Vision</I18N></Link>
                                    </li>
                                    <li>
                                        <Feature name="our-team">
                                            <Feature.Enabled>
                                                <Link endpoint='our-team'><I18N>Our Team</I18N></Link>
                                            </Feature.Enabled>
                                            <Feature.Disabled>
                                                <Link endpoint='info/view-path' values={{path: 'About/Staff'}}><I18N>Our Team</I18N></Link>
                                            </Feature.Disabled>
                                        </Feature>
                                    </li>
                                    <li>
                                        <Link endpoint='working-here'><I18N>Working Here</I18N></Link>
                                    </li>
                                    <li>
                                        <Link endpoint='info/view-path' values={{path: 'About/Content/Blogs'}}><I18N>Blogs</I18N></Link>
                                    </li>
                                    <li>
                                        <Link endpoint='info/view-path' values={{path: 'GradFairs'}}><I18N>Grad Fairs</I18N></Link>
                                    </li>
                                    <li>
                                        <Link endpoint='info/view-path' values={{path: this.helpPath()}}><I18N>Help</I18N></Link>
                                    </li>
                                    <li>
                                        <Link endpoint="contact-us"><I18N>Contact us</I18N></Link>
                                    </li>
                                    <li>
                                        <Link endpoint="donate"><I18N>Donate</I18N></Link>
                                    </li>
                                </ul>
                            </nav>
                            <div className="legalese">
                                <nav className="legal">
                                    <ul>
                                        <li><Link endpoint='info/view-path' values={{path: 'About/Terms'}}><I18N>Terms &amp; Conditions</I18N></Link></li>
                                        <li><Link endpoint='info/view-path' values={{path: 'About/Privacy'}}><I18N>Privacy Policy</I18N></Link></li>
                                        <li><Link endpoint='info/view-path' values={{path: 'Help/Guidelines'}}><I18N>Community Guidelines</I18N></Link></li>
                                    </ul>
                                </nav>
                                <span className="copyright"><I18N>
                                    ©1995-{currentYear}, Action Without Borders
                                    <span i18n-id="mobile-hide" className="mobile-hide">— a 501(c)3 non-profit organization.</span>
                                </I18N></span>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </footer>;
    }
});

module.exports = PageFooter;
