"use strict";

var _ = require('underscore');
var React = require('react/addons');
var classNames = require('classnames');
var ReactIntl = require('react-intl');

var App = require('dealers/App');
var LoginSignup = require('dealers/LoginSignup');
var NavMenu = require('dealers/NavMenu');
var Stats = require('dealers/Stats');

var Actions = require('Actions');
var IDOPropTypes = require('IDOPropTypes');
var Router = require('Router');
var Wish = require('Wish');
var World = require('World');
var {I18N, i18n} = require('I18N');

var Link = require('elements/Link');
var Look = require('elements/Look');
var PageHeaderAssetSearches = require('./PageHeaderAssetSearches');
var Search = require('./PageHeaderSearch');
var PageHeaderUserMenu = require('./PageHeaderUserMenu');
var SystemMessages = require('./SystemMessages');


var DesktopSubHeader = React.createClass({
    propTypes: {
        onPostListing: React.PropTypes.func.isRequired,
        orgCountWish: IDOPropTypes.immutable.map
    },
    render() {
        return <div className="susify full-size">
            <Search.PageHeaderSearch id="searchBox" />
            <div className="site-actions">
                <a className="button post-listing" onClick={this.props.onPostListing} href="#">
                    <I18N>Post a listing</I18N>
                </a>
            </div>
            <Look.Quick wish={this.props.orgCountWish}>
                {orgCount => <span className="org-count-msg">
                    <I18N><span className="org-count" i18n-id="orgcount"><ReactIntl.FormattedNumber value={orgCount}/></span> organizations use Idealist.</I18N>
                </span>}
            </Look.Quick>
        </div>;
    }
});


var MobileSubHeader = React.createClass({
    propTypes: {
        onPostListing: React.PropTypes.func.isRequired
    },
    render() {
        return <div className="susify mobile-size">
            <a href="#footer-search"><I18N>Search</I18N></a>
            <a className="post-listing" onClick={this.props.onPostListing} href="#"><I18N>Post</I18N></a>
            <a href="#footer-more"><I18N>More</I18N></a>
        </div>;
    }
});


Actions.register('PageHeader/RedirectIfAuthenticated', (route) => {
    return {
        effect: (world) => {
            Wish.look(App.currentUser(world), {
                Pending: () => {},
                Error: () => {},
                Available: (user) => {
                    user = user.toJS();
                    if (!user.unauthorized) {
                        document.location.href = Router.urlForRoute(route);
                    }
                }
            });
        }
    };
});


var PageHeader = React.createClass({
    mixins: [World.Child],

    getInitialState() {
        return {wantsToPostListing: false};
    },

    onPostListing(event) {
        event.preventDefault();
        this.setState({wantsToPostListing: true});
    },

    componentWillUpdate(nextProps, nextState) {
        if (!nextState.wantsToPostListing) {
            return;
        }
        var currentUser = App.currentUser(this.world());

        Wish.look(currentUser, {
            Pending: () => {},
            Available: (user) => {
                this.directUserToAddOrgOrCreateContent(user);
                _.defer(() => this.setState({wantsToPostListing: false}));
            },
            Error: () => {}
        });
    },

    directUserToAddOrgOrCreateContent(user) {
        var user = user.toJS();
        var createContent = Router.route('step/track/passthrough', {
            step: 'join-org',
            segment: '-',
            identifier: 'post',
            path: 'create-content'
        });
        var addOrg = Router.route('step/track/passthrough', {
            step: 'join-org',
            segment: '-',
            identifier: 'post',
            path: 'add-org'
        });
        if (user.known_admin && user.unauthorized) {
            LoginSignup.openSignup(this.world(), {
                name: 'PageHeader/RedirectIfAuthenticated',
                params: [createContent]
            });
        } else if (user.org_admin) {
            document.location.href = Router.urlForRoute(createContent);
        } else {
            document.location.href = Router.urlForRoute(addOrg);
        }
    },

    render: function () {
        var headerClasses = NavMenu.slidingClassNames(this.world());
        return <header className={classNames('ido-header', headerClasses)} role="banner">
            <div className="sup-header">
                <div className="susify">
                    <div className="brand">
                        <h1>
                            <span className="site-title hide-text"><I18N>Idealist.org</I18N></span>
                            <a href="/" title="Idealist.org">
                                <img className="idealist-logo" src="/images/logos/idealist.gif" width="180" alt="Idealist.org" />
                            </a>
                        </h1>
                    </div>
                    <a className="jumplink" href="#main" tabIndex="1"><I18N>Jump to content</I18N></a>
                    <PageHeaderAssetSearches />
                    <PageHeaderUserMenu />
                </div>
            </div>
            <div className="sub-header">
                <DesktopSubHeader
                    onPostListing={this.onPostListing}
                    orgCountWish={Stats.get(this.world(), 'orgcount')} />
                <MobileSubHeader onPostListing={this.onPostListing} />
            </div>
            <SystemMessages />
        </header>;
    }
});

module.exports = PageHeader;
