"use strict";

var _ = require('underscore');
var React = require('react/addons');

var App = require('dealers/App');
var Auth = require('dealers/Auth');
var Client = require('dealers/Client');
var LoginSignup = require('dealers/LoginSignup');
var NavMenu = require('dealers/NavMenu');

var ClippedNumber = require('elements/ClippedNumber');
var Link = require('elements/Link');
var Look = require('elements/Look');
var Wish = require('Wish');
var If = require('abc/if');
var PopupMenu = require('abc/popup_menu');
var World = require('World');
var {I18N, i18n} = require('I18N');

var AdministeredAssets = require('./AdministeredAssets');


var PageHeaderUserMenu = React.createClass({
    mixins: [World.Child],

    getInitialState: function() {
        return { showingSignin: false };
    },

    unauthedUserMenu: function () {
        return <ul>
            <li className="login" id="paLogIn">
                <button type="button" className="button" onClick={this.openSignup}><I18N>Log in</I18N></button>
            </li>
            <li className="signUp" id="paSignUp">
                <button type="button" className="button" onClick={this.openSignup}><I18N>Sign up</I18N></button>
            </li>
        </ul>;
    },

    authedUserMenu: function(currentUser) {
        var currentUser = currentUser.toJS();
        var administeredAssets = App.currentAdministeredAssets(this.world());

        var menuTriggerElement = <div id="me" className="current uActions">
            <Link className="me-menu" endpoint='me/view'>
                <span className="short-name">{currentUser.first_name}</span>&nbsp;
		{currentUser.inbox_count >0 && <span className="msgCount">
                    <ClippedNumber value={currentUser.inbox_count} />
		</span>}
            </Link>
        </div>;

        return <ul className="user-links">
            {!Client.isMobile(this.world())
            ?
                <li id="currentUser">
                    <PopupMenu dropTab={true} className="me-menu-body menu-items scrolling" trigger={menuTriggerElement}>
                        {/* Account links / meArea*/}
                        <ul id="meArea" className="user-navlinks">
                            <li id="myProfileLink" className="dropListItem">
                                <Link endpoint='me/view'>
                                    <figure className="avatar-icon">
					<img src={currentUser.profile_img_url || '/images/user/noPhoto-t.gif'}
                                             alt={currentUser.full_name}
                                             className="my-avatar" />
                                        <figcaption className="profile-link"><I18N>Profile</I18N></figcaption>
                                    </figure>
                                </Link>
                            </li>
                            <li id="myInboxLink" className="dropListItem">
                                <Link endpoint='me/messages'>Inbox ({currentUser.inbox_count})</Link>
                            </li>
                            <li id="myDashboardLink" className="dropListItem">
                                <Link endpoint='me/dashboard'><I18N>Dashboard</I18N></Link>
                            </li>
                        </ul> {/* end meArea */}

                        {/* Orgs */}
                        <Look.Quick wish={administeredAssets}>
                            {(assets) => <AdministeredAssets assets={assets} />}
                        </Look.Quick>{/* TODO: let the user know there was an error */}

                        <span id="logout" className="dropListItem log-out">
                            <Link className="logout" endpoint='logout'><I18N>Log out</I18N></Link>
                        </span>
                    </PopupMenu>
                </li> /* end currentUser */
            :
                <a id="nav-toggle" className="me-menu" href="#" onClick={this.toggleMobileNavMenu}>
                    <i className="navicon"></i>
                    <span className="short-name">{currentUser.first_name}</span>
                </a>
            }
        </ul>;
    },

    toggleMobileNavMenu(event) {
        event.preventDefault();
        NavMenu.toggleMobileNavMenu(this.world());
    },

    render: function () {
        var currentUser = App.currentUser(this.world());

        return <nav className="user-nav" role="navigation">
            <Look wish={currentUser}>
                {{
                    Available: function(value) {
                        if (Auth.isUnauthed(this.world())) {
                            return this.unauthedUserMenu(value);
                        }
                        return this.authedUserMenu(value);
                    }.bind(this),
                    Pending: () => null,
                    Error: this.unauthedUserMenu
                }}
            </Look>
        </nav>;
    },

    openSignup: function() {
        LoginSignup.openSignup(this.world());
    }
});

module.exports = PageHeaderUserMenu;
