"use strict";

var _ = require('underscore');
var React = require('react/addons');

var {i18n} = require('I18N');

// HOW TO USE PANEFLOW:
// <PaneFlow flow = {[FirstPane, [SecondPane, {prop:'Foo', ... }], ... ]}
//           currentPane = {2}
//           onChangePane = fn(paneIndex) => {...}
//           backButtonText = 'Retreat'     // OPTIONAL
//           nextButtonText = 'Advance' />  // OPTIONAL

var type = React.PropTypes;

var PaneFlow = React.createClass({
    propTypes: {
        flow: type.arrayOf(
            type.oneOfType(
                [type.func, //ReactComponentConstructor
                 type.arrayOf(
                    type.oneOfType(
                        [type.func, type.object]))])).isRequired,
        backButtonText: type.string,
        nextButtonText: type.string,
        currentPane: type.number.isRequired,
        onChangePane: type.func.isRequired,
        override: type.func
    },

    paneAtIndex: function(index) {
        var item = this.props.flow[index];
        return _.isArray(item) ? item[0] : item;
    },

    passedPaneProps: function(index) {
        var item = this.props.flow[index];
        return _.isArray(item) ? item[1] : {};
    },

    // event handlers passed down to the PaneNavButtons component on each pane
    goBack: function() { this.props.onChangePane(this.index() - 1); },
    goNext: function() { this.props.onChangePane(this.index() + 1); },

    render: function() {
        var pane = this.props.override || this.paneAtIndex(this.index());
        var paneProps = {...this.passedPaneProps(this.index()), ...this.props,
            atFlowStart: (this.index() < 1),
            atFlowEnd: (this.index() > this.props.flow.length - 2),
            goBack: this.goBack,
            goNext: this.goNext,
            ref: "pane",
            valid: true  // validation is optional for pane components
        };
        return React.createElement(pane, paneProps);
    },

    index: function() {
        return this.props.currentPane;
    },

    focus: function() {
        var pane = this.refs.pane;
        if (pane.focus) {
            pane.focus();
        }
    }
});

// WHEN WRITING PANE COMPONENTS:
// put a <PaneNavButtons {... this.props} /> in each pane's render method
var PaneNavButtons = React.createClass({
    render: function () {
        return <p className="actions">
            { this.props.atFlowStart ? '' :
                <a href="#" onClick={this.props.goBack} className='back'>
                    { this.props.backButtonText || i18n('‹‹ Back') }
                </a> }

            { this.props.atFlowEnd ? '' :
                <button onClick={this.props.valid ? this.props.goNext : this.props.onInvalid}
                        className={'next' + (this.props.valid ? '' : ' disabled')}>
                    { this.props.nextButtonText || i18n('Next ››') }
                </button> }
        </p>;
    }
});

module.exports = {PaneFlow, PaneNavButtons};
