var _ = require('underscore');

var Valid = require('abc/validators');


module.exports = {
    /*
    this.validators should look like:

    validators: {
        email: () => Valid.emailAddress(Signin.getField('email')),
        full_name: () => Valid.notEmpty(Signin.getField('full_name')),
        first_name: () => Valid.lengthInRange(Signin.getField('first_name'), 1, 128),
        location: () => Valid.location(Signin.getField('location'))
    }

    super important that those are functions. they could be anything that returns
    {ok:bool, errorMsg:str [, children:array(objects like this one)]}.
    */

    getInitialState() {
        if (!this.validators) {
            return;
        }
        return this._allValidationGates(false);
    },

    _allValidationGates(defaultVal) {
        var validationGates = {};
        for (var validatorName in this.validators) {
            validationGates[validatorName] = defaultVal;
        }
        return {validationGates};
    },

    _changeFieldValidation(fld, val) {
        return (evt) => {
            var newGates = _.clone(this.state.validationGates);
            newGates[fld] = val;
            this.setState({validationGates: newGates});
        };
    },

    showFieldValidation(fld) {
        return this._changeFieldValidation(fld, true);
    },

    hideFieldValidation(fld) {
        return this._changeFieldValidation(fld, false);
    },

    showAllFieldValidation() {
        this.setState(this._allValidationGates(true));
    },

    hideAllFieldValidation() {
        this.setState(this._allValidationGates(false));
    },

    showValid: function(which) {
        return Valid.gated(this.state.validationGates[which],
            this.validators[which]());
    }
};
