'use strict';

var React = require('react');
var _ = require('underscore');
var Logger = require('Logger');

var WysiHtmlEditor = React.createClass({
    propTypes: {
        value: React.PropTypes.string,
        toolbarId: React.PropTypes.string.isRequired,
        parserRules: React.PropTypes.object.isRequired,
        stylesheetUrl: React.PropTypes.string.isRequired
    },

    render: function() {
        return <textarea
            value={this.props.value}
            onChange={function() {}}
            className="rich-text-editor" />;
        /* Uses a no-op onChange to suppress React's warning, because React
        doesn't see that we are listening for changes using WYSIHTML5's events. */
    },

    componentDidMount: function() {
        this._rte = new wysihtml5.Editor(React.findDOMNode(this),
                                         this.getEditorOptions());
        if (this._rte.config.toolbar) {
            this.$toolbar = $('#' + this._rte.config.toolbar);

        }

        this._addCommands();
        this._bindEvents();
    },

    _addCommands: function() {
        this.registerUploadCareCommand();
        this.registerInsertRulerCommand();
        this.registerEmbedYoutubeCommand();
    },

    registerEmbedYoutubeCommand: function () {
        // Custom wysihtml5 command to embed YouTube videos
        var self = this;
        wysihtml5.commands.embedYoutube = {
            exec: function (composer, command, value) {
                value.src = value.src || '';
                value.width = value.width || '';

                var embedUrl = self.validateAndBuildEmbedLink(value.src);
                if (!embedUrl) {
                    return;
                }
                var video = composer.doc.createElement('IFRAME');
                video.src = embedUrl;
                video.width = value.width;
                video.height = self.videoAspectRatio.heightForWidth(value.width);
                composer.selection.editor.currentView.focus();
                composer.selection.insertNode(video);
                composer.selection.setAfter(video);
            }
        };
    },

    registerUploadCareCommand: function () {
        // Custom wyishtml5 command to insert UploadCare image
        var self = this;
        wysihtml5.commands.insertUploadCare = {
            exec: function (composer, command) {
                var dialog = uploadcare.openDialog();
                var caretBookmark = composer.selection.getBookmark();
                dialog.then(function(filePromise) {
                    composer.selection.setBookmark(caretBookmark);
                    composer.selection.editor.currentView.element.focus();
                    var whirly = '<img src="/images/ajax.gif" class="whirly" data-idealist-editor-transient="yes" />';
                    wysihtml5.commands.insertHTML.exec(composer,
                                                       'insertHTML',
                                                        whirly);
                    return filePromise;
                }).then(function(fileInfo) {
                    self.restrictImageSize(fileInfo);
                    var preload = new Image();
                    var modifiers = fileInfo.cdnUrlModifiers ? fileInfo.cdnUrlModifiers : '';
                    preload.onload = function() {
                        var whirly = composer.element.querySelector('.whirly');
                        whirly.parentNode.removeChild(whirly);
                        composer.selection.setBookmark(caretBookmark);
                        composer.selection.editor.currentView.element.focus();
                        wysihtml5.commands.insertImage.exec(composer, 'insertImage', this.src);

                        // WYSIHTML5 helpfully doesn't return the inserted image, so find it.
                        // The map from (provider,uri) to src is injective, so this is safe.
                        var newImage = _.filter(composer.element.getElementsByTagName('img'),
                            function (img) {
                                return img.src.search(fileInfo.cdnurl) === 0;
                            })[0];
                        newImage.setAttribute("data-idealist-editor-upload-uri", fileInfo.cdnUrl);
                        newImage.setAttribute("data-idealist-editor-upload-provider", "uploadcare");
                    };
                    preload.src = fileInfo.cdnUrl + modifiers;
                });
            }
        };
    },

    registerInsertRulerCommand: function () {
        // Custom wysihtml5 command to insert an <hr>
        wysihtml5.commands.insertRuler = {
            exec: function (composer, command) {
                wysihtml5.commands.insertHTML.exec(composer,
                                                   'insertHTML',
                                                   '<hr>');
            }
        };
    },

    _bindEvents: function() {
        // check if link text field mirrors url on url keystroke
        this.mirrorLinkText = false;

        // Image resizing
        this.bindResizeImageEvent();
    },

    getEditorOptions: function() {
        return {
            toolbar: this.props.toolbarId,
            parserRules: this.props.parserRules,
            stylesheets: [this.props.stylesheetUrl]
        };
    },

    bindResizeImageEvent: function () {
        var self = this,
            currentSize;
        uploadcare.registerTab('resize', resizeImage);
        $(self._rte.composer.iframe).contents().find('body')
            .on('click', 'img:not(.whirly)', function(evt, ui) {
                var targetFilePromise = uploadcare.fileFrom('uploaded', this.src);
                uploadcare.openDialog(null, null, {
                    tabs: 'resize',
                    filePromise: targetFilePromise,
                    targetImage: this
                });
            });

        // UploadCare API custom image resizer dialog
        function resizeImage (container, button, dialogApi, settings) {
            var targetFilePromise = settings.filePromise;
            var targetImage = settings.targetImage;

            if (!_.isUndefined(targetFilePromise)) {
                targetFilePromise.done(function (fileInfo) {
                    resizeTabSetup(container, dialogApi, targetImage, fileInfo);
                });
                targetFilePromise.fail(function () {
                    Logger.error('Error retrieving data for UploadCare image');
                    dialogApi.done();
                });
            }
        }

        // initilize resize tab elements, events
        function resizeTabSetup ($container, dialogApi, targetImage, targetImageInfo) {
            var $clone = $('<img>').attr('src', targetImageInfo.originalUrl);
            var changes = [];   // list of undoable resizes
            $container.show(100, function () {
                var $title = $('.uploadcare-dialog-title').first().clone();
                $container.append($title.clone());
                var $undo = self.$toolbar.find('[data-resize-image=undo]').clone();
                $undo.prop('disabled', true)
                    .on('click', function (evt) {
                        evt.preventDefault();
                        if (changes.length) {
                            var size = changes.pop();
                            setResize($clone, size);
                            currentSize = size;
                        }
                        if (changes.length === 0) {
                            $(this).prop('disabled', true);
                        }
                    });
                var $complete = self.$toolbar.find('[data-resize-image=complete]').clone();
                $complete.on('click', function (evt) {
                    evt.preventDefault();
                    if (typeof targetImage !== 'undefined' &&
                        typeof targetImageInfo !== 'undefined' &&
                        typeof currentSize !== 'undefined') {
                        self.applyResizeImage(targetImage, targetImageInfo, currentSize);
                    }
                    dialogApi.resolve();
                });
                var $resizeArea = $('<div>').addClass('resize-area');
                $container.append($resizeArea);
                $resizeArea.append($clone);
                $clone.resizable({
                    handles: 'se',
                    aspectRatio: true,
                    containment: 'parent',
                    start: function (evt, ui) {
                        changes.push(ui.originalSize);
                        $undo.prop('disabled', false);
                    },
                    stop: function (evt, ui) {
                        currentSize = ui.size;
                    }
                });
                setResize($clone, targetImage, $resizeArea);
                $container.append($undo).append($complete);
            });
        }

        // convenience function for specifying size for jquery-ui-resizable
        // if parentElement is specified, fit image within its dimensions
        function setResize (resizable, size, parentElement) {
            var ratio = 1;
            if (typeof parentElement !== 'undefined') {
                if (size.width > size.height &&
                    size.width > parentElement.width()) {
                    ratio = parentElement.width() / size.width;
                } else if (size.height > size.width &&
                           size.height > parentElement.height()) {
                    ratio = parentElement.height() / size.height;
                }
            }
            resizable.width(size.width * ratio);
            resizable.parents('.ui-wrapper').width(size.width * ratio);
            resizable.height(size.height * ratio);
            resizable.parents('.ui-wrapper').height(size.height * ratio);
        }
    },

    applyResizeImage: function(image, fileInfo, size) {
        var width = Math.round(size.width),
            height = Math.round(size.height);
        var resizePattern = /-\/resize\/(?:x\d+|\d+x\d*)\//;
        var resizeModifier = '-/resize/' + (width !== 'undefined' ? width : '')
                + 'x' + (height !== 'undefined' ? height : '') + '/';
        var cdnUrlModifier = '';
        if (fileInfo.hasOwnProperty('cdnUrlModifiers') &&
            fileInfo.cdnUrlModifiers !== null) {
            if (fileInfo.cdnUrlModifiers.match(resizePattern)) {
                cdnUrlModifier = fileInfo.cdnUrlModifiers.replace(resizePattern, resizeModifier);
            } else {
                cdnUrlModifier = fileInfo.cdnUrlModifiers + resizeModifier;
            }
        } else {
            cdnUrlModifier = resizeModifier;
        }
        image.src = fileInfo.originalUrl + cdnUrlModifier;
    },

    restrictImageSize: function (imageInfo) {
        var width = imageInfo.originalImageInfo.width,
            height = imageInfo.originalImageInfo.height,
            widthRatio = width / this.maxImageSize.width,
            heightRatio = height / this.maxImageSize.height;

        if (widthRatio >= heightRatio && widthRatio > 1) {
            imageInfo.cdnUrlModifiers = '-/resize/' + this.maxImageSize.width + 'x/';
        } else if (heightRatio > widthRatio && heightRatio > 1) {
            imageInfo.cdnUrlModifiers = '-/resize/x' + this.maxImageSize.height + '/';
        }
    },

    // UploadCare widget resize area size
    maxImageSize: {
        width: 775,
        height: 458
    },

    videoAspectRatio: {
        width: 16,
        height: 9,
        heightForWidth: function(width) {
            return Math.floor(width * this.height / this.width);
        }
    },

    validateAndBuildEmbedLink: function(link) {
        // as congealed from http://goo.gl/LSMCrL
        var ytRegex = /(http(?:s?):)?\/\/(?:www\.)?youtu(?:be\.com\/(watch\?v=|embed\/)|\.be\/)([\w\-\_]*)(&(amp;)?[\w\?=]*)?/;
        var matched;

        // grab the video id and stick it into the embed link format
        matched = link.match(ytRegex);
        if (matched && matched[3].length === 11) {
            // youtube obfuscates all video ids to 11 characters
            return '//www.youtube.com/embed/' + matched[3] + '?wmode=transparent';
        } else {
            $.modal.close();
            // TODO:dh field errors?
            return false;
        }
    }
});

module.exports = WysiHtmlEditor;
