var React = require('react');
var {Spring, TransitionSpring, presets} = require('react-motion');
var SpringPresets = presets;
var _ = require('underscore');
var Immutable = require('immutable');
var ReactIntl = require('react-intl');

var config = require('config');

var API = require('API');
var App = require('dealers/App');
var Auth = require('dealers/Auth');
var Campaign = require('dealers/Campaign');
var Client = require('dealers/Client');
var Stats = require('dealers/Stats');
var DOM = require('DOM');
var LoginSignup = require('dealers/LoginSignup');
var Router = require('Router');
var Wish = require('Wish');
var World = require('World');
var {I18N, i18n} = require('I18N');

var LegacyI3APIHeaders = require('legacy/I3APIHeaders');

var Link = require('elements/Link');
var Look = require('elements/Look');
var UserIntake = require('elements/OrgIntake/UserIntake');
var OrgIntakeCore = require('elements/OrgIntake/OrgIntakeCore');
var OrgIntakeForm = require('dealers/OrgIntakeForm');
var Spinner = require('elements/Spinner');

var Org = require('schemas/Org');


var Promo = React.createClass({
    mixins: [World.Child],

    getInitialState() {
        return {
            arrowOpacity: 1.0,
            jumping: false,
            finalScroll: null
        };
    },

    componentDidMount() {
        this._startingArrowHeight = DOM.offsetTop(React.findDOMNode(this.refs.scrollArrow));
        this._scrollHandler = this.fadeArrow;
        if (window.addEventListener) {
            window.addEventListener('scroll', this._scrollHandler);
        } else {
            window.attachEvent('onscroll', this._scrollHandler);
        }
        this.fadeArrow();
    },

    componentWillUnmount() {
        if (window.addEventListener) {
            window.removeEventListener('scroll', this._scrollHandler);
        } else {
            window.detachEvent('onscroll', this._scrollHandler);
        }
    },

    PROMO_SUFFIXES: {
        'en': 'en_us',
        'es': 'es',
        'fr': 'fr',
        'default': 'en_non_us'
    },

    promo() {
        var suffix = this.PROMO_SUFFIXES[this.world().get('Locale')] || this.PROMO_SUFFIXES['default'];
        var url = config.idealist['org_intake.promo.' + suffix];
        var hasVideo = /youtu(\.)?be/.test(url);
        return {
            url: url + (hasVideo ? '?wmode=transparent&rel=0&autohide=1&showinfo=0' : ''),
            hasVideo: hasVideo
        };
    },

    fadeArrow() {
        //get scroll position
        var topWindow = DOM.scrollOffsetTop();
        //multipl by 1.5 so the arrow will become transparent half-way up the page
        var topWindow = topWindow * 1.5;

        //set position as percentage of how far the user has scrolled
        var position = topWindow / this._startingArrowHeight;
        //invert the percentage
        position = 1 - position;

        //define arrow opacity as based on how far up the page the user has scrolled
        //no scrolling = 1, half-way up the page = 0
        if (position >= 0) {
            this.setState({aboveFoldPosition: position});
        }
    },

    arrowJump(event) {
        event.preventDefault();
        var initialTime = Date.now();
        var initialScroll = window.scrollY;
        var finalScroll = DOM.offsetTop(document.querySelector('#start-org-intake'));

        this.setState({
            jumping: true,
            finalScroll
        });
    },

    render() {
        var dailyVisitors = 100000;
        var opportunityCount = 30000;
        var promo = this.promo();

        function getVertical() {
            // this exists at all because IE; it's a function because node
            return typeof window.scrollY !== 'undefined' ? window.scrollY : document.body.parentNode.scrollTop;
        }

        return <section className="intro susify">
            <div className="block">
                <div className="message">
                    <h1 className="title"><I18N>Share your opportunities.<br/>Find the right people.</I18N></h1>
                    <p><I18N>Recruit staff, volunteers, and supporters. Promote your programs and services to the millions who visit us each month.</I18N></p>
                </div>

                {promo.hasVideo
                ? <div className="promo-content">
                    <iframe src={promo.url} frameBorder="0" allowFullScreen="true"></iframe>
                </div>
                : <div className="promo-content image-fallback">
                    <img src={promo.url} />
                </div>}

                <ul className="stats">
                    <li><I18N><span i18n-id="stat" className="stat"><ReactIntl.FormattedNumber value={dailyVisitors}/></span>daily visitors</I18N></li>
                    <li><I18N><span i18n-id="stat" className="stat">{this.props.orgCount}</span>organizations</I18N></li>
                    <li><I18N><span i18n-id="stat" className="stat"><ReactIntl.FormattedNumber value={opportunityCount}/></span>opportunities</I18N></li>
                </ul>

                <a id="scroll-arrow" className="arrow-wrap" href="#start-org-intake"
                    style={{opacity: this.state.aboveFoldPosition}}
                    onClick={this.arrowJump}
                    ref="scrollArrow">
                    <span className="arrow"></span>
                </a>

                {this.state.jumping && <Spring defaultValue={{val: getVertical()}} endValue={{val: this.state.finalScroll}}>
                    {interpolated => {
                        let nextVal = interpolated.val;
                        if (Math.abs(interpolated.val - this.state.finalScroll) < 1) {
                            setImmediate(() => this.setState({jumping: false}));
                            nextVal = this.state.finalScroll;
                        }
                        window.scrollTo(0, nextVal);
                        return null;
                    }}
                </Spring>}
            </div>
        </section>;
    }
});


var OrgIntakePricing = React.createClass({
    render: function () {
        return <Look wish={this.props.markup}>
            {{Pending: () => <div key="pending" className="pricing paid">
                <h2><I18N>What kind of opportunities can my org post?</I18N></h2>
                <p style={{textAlign: 'center', margin: 80}}>
                    <Spinner />
                </p>
            </div>,
            Error: () => <span>Error</span>,
            Available: (markup) =>
                <div key="available" dangerouslySetInnerHTML={
                    {__html: markup.getIn(['updates', 'body'])}
                }/>
            }}
        </Look>;
    }
});


var OrgLogos = React.createClass({
    mixins: [World.Child],

    logos : {
        'en': [
            ['World Food Programme', '/images/org/org-intake/en/wfp.png'],
            ['American Red Cross', '/images/org/org-intake/en/american-red-cross.png'],
            ['City Year', '/images/org/org-intake/en/city-year.png'],
            ['Amnesty International', '/images/org/org-intake/en/amnesty-international.png'],
            ['OXFAM', '/images/org/org-intake/en/oxfam.png'],
            ['International Federation', '/images/org/org-intake/en/international-federation.png'],
            ['Sierra Club', '/images/org/org-intake/en/sierra-club.png']
        ],
        'es': [
            ['Ashoka Changemakers', '/images/org/org-intake/es/ashoka-changemakers.png'],
            ['World Food Programme', '/images/org/org-intake/en/wfp.png'],
            ['Amnesty International', '/images/org/org-intake/en/amnesty-international.png'],
            ['OXFAM', '/images/org/org-intake/en/oxfam.png'],
            ['TECHO', '/images/org/org-intake/es/techo.png'],
            ['International Federation', '/images/org/org-intake/en/international-federation.png'],
            ['Wingu | Tecnología Sin Fines De Lucro', '/images/org/org-intake/es/wingu.png']
        ],
        'fr': [
            ['World Food Programme', '/images/org/org-intake/en/wfp.png'],
            ['Amnesty International', '/images/org/org-intake/en/amnesty-international.png'],
            ['OXFAM', '/images/org/org-intake/en/oxfam.png'],
            ['International Federation', '/images/org/org-intake/en/international-federation.png'],
            ['Compter sur demain', '/images/org/org-intake/fr/compter-sur-demain.png'],
            ['American Red Cross', '/images/org/org-intake/en/american-red-cross.png'],
            ['L\'Arche à Paris', '/images/org/org-intake/fr/larche.png']
        ]
    },

    render() {
      return <div>
        <section className="org-logos ">
          <h1 className="block-title susify">
              <I18N>Join more than {this.props.orgCount} organizations from around the world</I18N>
          </h1>
          <div className="susify">
            <div className="block">
              <ul>
                  {this.logos[this.world().get('Locale')].map(([alt, url]) =>
                      <li key={url}><img src={url} alt={alt} title={alt} /></li>
                  )}
              </ul>
            </div>
          </div>
        </section>
      </div>;
    }
});


var OrgIntake = React.createClass({
    mixins: [World.Child],

    statics: {
        htmlHead: () => {
            return {
                title: i18n('Add Your Org'),
                stylesheet: 'org-intake'
            };
        },

        pageWrapperFeatures: {
            'has-post-msg': true
        }
    },

    getEndValue() {
        return Auth.isDefinitelyUnauthed(this.world())
            ? {
                intake: {
                    top: {val: 0},
                    opacity: {val: 1}
                }
            }
            : {};
    },

    willLeave() {
        return {
            top: {val: this._userIntake ? -this._userIntake.clientHeight : 0},
            opacity: {val: 0, config: SpringPresets.stiff}
        };
    },

    render() {
        var authed = !!Auth.isDefinitelyAuthed(this.world());
        var pendingVerified = Auth.emailAddressPendingVerification(this.world());

        var pricingMarkup = OrgIntakeForm.pricingMarkup(this.world());

        var orgCountDisplay = <Look.Quick wish={Stats.get(this.world(), 'orgcount')}>
            {(orgCount) => <ReactIntl.FormattedNumber value={Math.floor(orgCount / 1000) * 1000}/>}
        </Look.Quick>;

        return <div>
            <Promo orgCount={orgCountDisplay} />

            <section className="how-to-post" id="start-org-intake">
              <h1 className="block-title susify"><I18N>How do I post an opportunity on Idealist?</I18N></h1>
              <div className="full-width">
                <ul className="three-steps susify">
                  <li className={!authed && 'current-step step-1'}>
                    <I18N><h2>1. Create a personal account</h2>
                    <p>It only takes a second to create your login and password. And it’s free!</p></I18N>
                  </li>
                  <li className={authed && 'current-step step-2'}>
                    <I18N foo="bar"><h2>2. Add your organization</h2>
                    <p>You’re doing awesome work. Tell the world! (This is also free.)</p></I18N>
                  </li>
                  <li>
                    <I18N><h2>3. Post away!</h2>
                    <p>There are tons of idealists looking to help your cause. Help them find you.</p></I18N>
                  </li>
                </ul>
              </div>
            </section>

            <TransitionSpring endValue={this.getEndValue} willLeave={this.willLeave}>
                {(interpolated) => {
                    let translate = `translateY(${interpolated.intake ? interpolated.intake.top.val : 0}px)`;
                    return <div
                        style={{transform:           translate,
                                'WebkitTransform':   translate,
                                'msTransform':       translate,
                                'OTransform':        translate}}>

                        {interpolated.intake
                            && !LoginSignup.isModalSignupOpen(this.world())
                            && <UserIntake ref={(component) => this._userIntake = React.findDOMNode(component)}
                                    opacity={interpolated.intake.opacity.val}
                                    pendingVerified={pendingVerified}/>}

                        {/* Org Intake */}
                        <section className="org-intake">
                          {authed
                           ? <h1 className="block-title susify step-2"><I18N><span i18n-id="step-2" className="step-text">Step 2: </span>Add your organization to Idealist</I18N></h1>
                           : <h1 className="block-title susify what-kind"><I18N>What kind of opportunities can my org post?</I18N></h1>}
                          <div className="susify">
                            {authed && <p>
                                <I18N>Is your organization already on Idealist? <a href="#root">Search to check first</a></I18N>.&nbsp;
                                <Look wish={App.currentAdministeredAssets(this.world())}>{{
                                    Pending: () => null,
                                    Error: () => null,
                                    Available: assets => {
                                        return (
                                            assets.get('orgs').size ?
                                            <I18N>Already an administrator of your organization account? <a href="/create-content">Start posting</a>.</I18N> :
                                            null
                                        );
                                    }
                                }}</Look>
                            </p>}
                            <div className="block" id="org-intake-component">
                                <OrgIntakeCore authed={authed} />
                                <OrgIntakePricing markup={pricingMarkup} />
                            </div>
                          </div>
                        </section>

                        <OrgLogos orgCount={orgCountDisplay} />
                    </div>;}
                }
            </TransitionSpring>
        </div>;
    },

    updatePromo(success, changedField) {
        if (changedField.name === 'type' ||
            changedField.name === 'addresses' ||
            changedField === OrgIntakeForm.state) {
                this.forceUpdate();
        }
    },

    componentWillMount() {
        OrgIntakeForm.initForm(this);
        OrgIntakeForm.state.observe(this.updatePromo);
    },

    componentDidMount() {
        let actions = [];
        let activationAction = Client.queryParams().get('activation_action');
        if (activationAction) {
            actions.push(activationAction);
        }

        let campaign = Campaign.fromQueryParams();
        if (campaign) {
            actions.push(Campaign.activationAction(campaign));
        }

        OrgIntakeForm.setField('activationActions', actions);
    },

    componentWillUnmount() {
        OrgIntakeForm.state.unobserve(this.updatePromo);

        // unsetting flag set in SignupSecure when the inline signup flow is used
        this.world().performAction({
            name: 'SignupFlow.componentWillUnmount',
            transaction: (ws) => ws.deleteIn(['Auth', 'hideEmailVerificationMsg'])
        });
    }
});


module.exports = OrgIntake;
