"use strict";

var React = require('react');
var Link = require('elements/Link');
var LocationField = require('elements/LocationField');
var {Map} = require('immutable');

var If = require('abc/if');

var App = require('dealers/App');
var Auth = require('dealers/Auth');
var Playthings = require('dealers/Playthings');

var Autocomplete = require('elements/Autocomplete');

var Wish = require('Wish');
var World = require('World');

function isEmail(s) {
    return 0 < s.indexOf('@') && s.indexOf('@') < s.indexOf('.') && s.indexOf('.') < s.length - 1;
}

var InitialLoginSignupScreen = React.createClass({
    mixins: [World.Child],

    getInitialState: function () {
        return {
            email: ''
        };
    },

    render: function () {
        return <div>
            <h1>Sign in:</h1>
            <button onClick={this.props.go('social')}>G+</button>
            <button onClick={this.props.go('social')}>FB</button>
            <h2>Or use your email address:</h2>
            <input
                value={this.state.email}
                onChange={(e) => this.setState({'email': e.target.value})} />

            <If test={this.canAdvanceEmail()}>
                <button onClick={this.props.go(this.emailWhereToGo(), Map({email: this.state.email}))}>Next</button>
            </If>
        </div>;
    },

    canAdvanceEmail: function () {
        return isEmail(this.state.email) &&
        Wish.look(Auth.userExistsWithEmail(this.world(), this.state.email), {
            Available: () => true,
            Pending: () => false,
            Error: () => false
        });
    },

    emailWhereToGo: function () {
        return Wish.look(Auth.userExistsWithEmail(this.world(), this.state.email), {
            Available: (exists) => exists ? 'log-in-by-email' : 'sign-up-by-email',
            Pending: () => false,
            Error: () => false
        });
    }
});


var LogInByEmailScreen = React.createClass({
    getInitialState: function () {
        return {
            password: ''
        };
    },

    componentDidMount: function () {
        this.refs.password.getDOMNode().focus();
    },

    render: function () {
        return <div>
            <h1>Log in</h1>
            <p>Welcome back! Please enter your password.</p>
            <label>
                Email address
                <input disabled={true} value={this.props.gatherings.get('email')} />
            </label>
            <label>
                Password
                <input
                    ref="password"
                    type="password"
                    value={this.state.password}
                    onChange={(e) => this.setState({'password': e.target.value})} />
            </label>
            <p>Forgot your password?</p>
            <button onClick={this.props.go('attempt-email-login', Map({password: this.state.password}))}>Next</button>
        </div>;
    }
});


var AttemptEmailLoginScreen = React.createClass({
    mixins: [World.Child],

    componentDidMount: function () {
        Auth.logInByEmail(this.world(),
            this.props.gatherings.get('email'),
            this.props.gatherings.get('password')
        );
    },

    render: function () {
        return Wish.look(App.currentUser(this.world()), {
            Error: () => <div>Error</div>,
            Pending: () => this.renderSpinner(),
            Available: (user) =>
                user.get('unauthorized') ?
                    this.renderSpinner() :
                    this.renderComplete()
        });
    },

    renderSpinner: function () {
        return <div>s p i n n e r</div>;
    },

    renderComplete: function () {
        return <div>done</div>;
    }
});


var Steps = React.createClass({
    render: function () {
        return React.createElement(this.props.steps[this.state.currentStep], {
            go: this.go,
            gatherings: this.state.gatherings
        });
    },

    go: function (transitionName, newGatherings) {
        return () => {
            this.setState({
                'currentStep': this.props.transitions[this.state.currentStep][transitionName],
                'gatherings': this.state.gatherings.merge(newGatherings)
            });
        };
    },

    getInitialState: function() {
        return {currentStep: 'initial', gatherings: Map({})};
    }
});


var LoginSteps = React.createClass({
    steps: function () {
        return {
            'initial': InitialLoginSignupScreen,
            'log-in-by-email': LogInByEmailScreen,
            'attempt-email-login': AttemptEmailLoginScreen
        };
    },

    transitions: function () {
        return {
            'initial': {
                'log-in-by-email': 'log-in-by-email',
                'sign-up-by-email': 'sign-up-by-email'
            },
            'log-in-by-email': {
                'attempt-email-login': 'attempt-email-login'
            }
        };
    },

    render: function () {
        return <Steps steps={this.steps()} transitions={this.transitions()} />;
    }
});


var Ugh = React.createClass({
    render: function() {
        return <button onClick={this.props.onClick}>Ugh!</button>;
    }
});

const TEST_DATA = ['Abercrombie','Basalt','Crumbly','Concern','Crandall','Zus'];

var TestWrapper = React.createClass({
    getInitialState() {
      return { results: [] };
    },

    handleInput(input) {
      this.setState({
          results: TEST_DATA.filter((res) => res.indexOf(input) === 0)
      });
    },

    render() {
      return <Autocomplete.Combobox
        ref="combobox"
        autocomplete="list"
        onInput={this.handleInput}>
        {this.state.results.map(
          result => <Autocomplete.Option value={result} />
        )}
      </Autocomplete.Combobox>;
    }
});

var Playground = React.createClass({
    mixins: [World.Child],

    statics: {
        htmlHead: function () {
            return {
                title: "Playground - Idealist.org",
                stylesheet: "playground"
            };
        },
        initialWishes: function (world) {
            return [];
        }
    },

    getInitialState: function(){
        return {location: LocationField.blankLocation()};
    },

    render: function() {
        return <div className="infopage susify">
            <h1>Playground</h1>
            <TestWrapper />
        </div>;
    }
});

module.exports = Playground;
