
var configCoersion = {
    'True': true,
    'False': false
};

/**
 * Grab a subset of an .ini config.
 *
 * Example whitelist:
 *
 * 		{
 * 			section1: [
 * 				'whitelisted-param-1',
 * 				'whitelisted-param-2',
 * 				/^whitelisted-params/
 * 			]
 * 		}
 *
 * @param  {Object} config    The ini config object.
 * @param  {Object} whitelist An object matching the shape of the ini config
 *                            object, where each section is an array of
 *                            Strings and/or RegExps.
 * @return {Object}           The subset of config options.
 */
function pluckConfig(config, whitelist) {
    return Object.keys(config).reduce(
        function(clientConfig, section) {
            // Only pull from sections contained in the whitelist
            clientConfig[section] = !whitelist[section] ? {} : whitelist[section].reduce(
                function(sectionConfig, key) {
                    return addSectionValue(sectionConfig, section, key);
                },
                {}
            );
            return clientConfig;
        },
        {}
    );

    function addSectionValue(sectionConfig, section, key) {
        if (typeof key === 'string') {
            sectionConfig[key] = configValue(section, key);
        }
        else {
            Object.keys(config[section]).forEach(function(sectionKey) {
                // console.log('config key', sectionKey);
                if (key.test(sectionKey)) {
                    sectionConfig[sectionKey] = configValue(section, sectionKey);
                }
            });
        }
        return sectionConfig;
    }

    function configValue(section, key) {
        var value = config[section][key];
        return (
            configCoersion.hasOwnProperty(value) ?
            configCoersion[value] :
            value
        );
    }
}

module.exports = pluckConfig;
