'use strict';

var {i18n} = require('I18N');

import {Schema, Validation} from 'conformist';
let {Int, Str, Bool, Map} = Schema;
let {Value} = Validation;

function fallbackValidator(desiredFallbackValue, message, test = (v) => !!v) {
  return function (element) {
    let actualFallbackValue = element.parent.members.fallback.value;
    if (actualFallbackValue === desiredFallbackValue) {
      if (!test(element.value)) {
        element.addError(message);
        return false;
      }
    }
    return true;
  };
}

function trimCheck(value) {
    let trimmed = value ? value.trim() : '';
    if (trimmed && trimmed.length) {
      return true;
    }
    return false;
}

let Location = Map.of(
  Bool.named('fallback').using({default: false}).validatedBy(Value.Present(i18n('Please choose a location'))),
  Str.named('type').validatedBy(fallbackValidator(false, i18n('Please choose a location'))),
  Str.named('location').validatedBy(fallbackValidator(false, i18n('Please choose a location'))),
  Str.named('formatted').validatedBy(fallbackValidator(false, i18n('Please choose a location'))),
  Int.named('geoid').validatedBy(fallbackValidator(false, i18n('Please choose a location'))),
  Str.named('city').validatedBy(
    fallbackValidator(true, i18n('Please provide a city name'), val => { return !!val && trimCheck(val); })
  ),
  Str.named('regionCode'),
  Str.named('countryCode').validatedBy(fallbackValidator(true, i18n('Please choose a country'))),
  Int.named('regionGeoid').validatedBy(fallbackValidator(true, i18n('Please choose a region'))),
  Int.named('countryGeoid').validatedBy(fallbackValidator(true, i18n('Please choose a country')))
);


export default Location;
